@@ -2,7 +2,7 @@
 // detail/handler_type_requirements.hpp
 // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2021 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -65,7 +65,7 @@ template <typename Handler>
 auto zero_arg_copyable_handler_test(Handler h, void*)
   -> decltype(
     sizeof(Handler(static_cast<const Handler&>(h))),
-    ((h)()),
+    (static_cast<Handler&&>(h)()),
     char(0));
 
 template <typename Handler>
@@ -74,8 +74,8 @@ char (&zero_arg_copyable_handler_test(Handler, ...))[2];
 template <typename Handler, typename Arg1>
 auto one_arg_handler_test(Handler h, Arg1* a1)
   -> decltype(
-    sizeof(Handler(BOOST_ASIO_MOVE_CAST(Handler)(h))),
-    ((h)(*a1)),
+    sizeof(Handler(static_cast<Handler&&>(h))),
+    (static_cast<Handler&&>(h)(*a1)),
     char(0));
 
 template <typename Handler>
@@ -84,8 +84,8 @@ char (&one_arg_handler_test(Handler h, ...))[2];
 template <typename Handler, typename Arg1, typename Arg2>
 auto two_arg_handler_test(Handler h, Arg1* a1, Arg2* a2)
   -> decltype(
-    sizeof(Handler(BOOST_ASIO_MOVE_CAST(Handler)(h))),
-    ((h)(*a1, *a2)),
+    sizeof(Handler(static_cast<Handler&&>(h))),
+    (static_cast<Handler&&>(h)(*a1, *a2)),
     char(0));
 
 template <typename Handler>
@@ -94,8 +94,9 @@ char (&two_arg_handler_test(Handler, ...))[2];
 template <typename Handler, typename Arg1, typename Arg2>
 auto two_arg_move_handler_test(Handler h, Arg1* a1, Arg2* a2)
   -> decltype(
-    sizeof(Handler(BOOST_ASIO_MOVE_CAST(Handler)(h))),
-    ((h)(*a1, BOOST_ASIO_MOVE_CAST(Arg2)(*a2))),
+    sizeof(Handler(static_cast<Handler&&>(h))),
+    (static_cast<Handler&&>(h)(
+      *a1, static_cast<Arg2&&>(*a2))),
     char(0));
 
 template <typename Handler>
@@ -114,13 +115,9 @@ template <typename T> T& lvref();
 template <typename T> T& lvref(T);
 template <typename T> const T& clvref();
 template <typename T> const T& clvref(T);
-#if defined(BOOST_ASIO_HAS_MOVE)
 template <typename T> T rvref();
 template <typename T> T rvref(T);
-#else // defined(BOOST_ASIO_HAS_MOVE)
-template <typename T> const T& rvref();
-template <typename T> const T& rvref(T);
-#endif // defined(BOOST_ASIO_HAS_MOVE)
+template <typename T> T rorlvref();
 template <typename T> char argbyv(T);
 
 template <int>
@@ -146,7 +143,7 @@ struct handler_type_requirements
           boost::asio::detail::clvref< \
             asio_true_handler_type>())) + \
       sizeof( \
-        boost::asio::detail::lvref< \
+        boost::asio::detail::rorlvref< \
           asio_true_handler_type>()(), \
         char(0))> BOOST_ASIO_UNUSED_TYPEDEF
 
@@ -171,7 +168,7 @@ struct handler_type_requirements
           boost::asio::detail::rvref< \
             asio_true_handler_type>())) + \
       sizeof( \
-        boost::asio::detail::lvref< \
+        boost::asio::detail::rorlvref< \
           asio_true_handler_type>()( \
             boost::asio::detail::lvref<const boost::system::error_code>(), \
             boost::asio::detail::lvref<const std::size_t>()), \
@@ -198,7 +195,7 @@ struct handler_type_requirements
           boost::asio::detail::rvref< \
             asio_true_handler_type>())) + \
       sizeof( \
-        boost::asio::detail::lvref< \
+        boost::asio::detail::rorlvref< \
           asio_true_handler_type>()( \
             boost::asio::detail::lvref<const boost::system::error_code>(), \
             boost::asio::detail::lvref<const std::size_t>()), \
@@ -224,7 +221,7 @@ struct handler_type_requirements
           boost::asio::detail::rvref< \
             asio_true_handler_type>())) + \
       sizeof( \
-        boost::asio::detail::lvref< \
+        boost::asio::detail::rorlvref< \
           asio_true_handler_type>()( \
             boost::asio::detail::lvref<const boost::system::error_code>()), \
         char(0))> BOOST_ASIO_UNUSED_TYPEDEF
@@ -250,7 +247,7 @@ struct handler_type_requirements
           boost::asio::detail::rvref< \
             asio_true_handler_type>())) + \
       sizeof( \
-        boost::asio::detail::lvref< \
+        boost::asio::detail::rorlvref< \
           asio_true_handler_type>()( \
             boost::asio::detail::lvref<const boost::system::error_code>(), \
             boost::asio::detail::rvref<socket_type>()), \
@@ -276,7 +273,7 @@ struct handler_type_requirements
           boost::asio::detail::rvref< \
             asio_true_handler_type>())) + \
       sizeof( \
-        boost::asio::detail::lvref< \
+        boost::asio::detail::rorlvref< \
           asio_true_handler_type>()( \
             boost::asio::detail::lvref<const boost::system::error_code>()), \
         char(0))> BOOST_ASIO_UNUSED_TYPEDEF
@@ -302,7 +299,7 @@ struct handler_type_requirements
           boost::asio::detail::rvref< \
             asio_true_handler_type>())) + \
       sizeof( \
-        boost::asio::detail::lvref< \
+        boost::asio::detail::rorlvref< \
           asio_true_handler_type>()( \
             boost::asio::detail::lvref<const boost::system::error_code>(), \
             boost::asio::detail::lvref<const endpoint_type>()), \
@@ -329,7 +326,7 @@ struct handler_type_requirements
           boost::asio::detail::rvref< \
             asio_true_handler_type>())) + \
       sizeof( \
-        boost::asio::detail::lvref< \
+        boost::asio::detail::rorlvref< \
           asio_true_handler_type>()( \
             boost::asio::detail::lvref<const boost::system::error_code>(), \
             boost::asio::detail::lvref<const iter_type>()), \
@@ -356,7 +353,7 @@ struct handler_type_requirements
           boost::asio::detail::rvref< \
             asio_true_handler_type>())) + \
       sizeof( \
-        boost::asio::detail::lvref< \
+        boost::asio::detail::rorlvref< \
           asio_true_handler_type>()( \
             boost::asio::detail::lvref<const boost::system::error_code>(), \
             boost::asio::detail::lvref<const range_type>()), \
@@ -382,7 +379,7 @@ struct handler_type_requirements
           boost::asio::detail::rvref< \
             asio_true_handler_type>())) + \
       sizeof( \
-        boost::asio::detail::lvref< \
+        boost::asio::detail::rorlvref< \
           asio_true_handler_type>()( \
             boost::asio::detail::lvref<const boost::system::error_code>()), \
         char(0))> BOOST_ASIO_UNUSED_TYPEDEF
@@ -408,7 +405,7 @@ struct handler_type_requirements
           boost::asio::detail::rvref< \
             asio_true_handler_type>())) + \
       sizeof( \
-        boost::asio::detail::lvref< \
+        boost::asio::detail::rorlvref< \
           asio_true_handler_type>()( \
             boost::asio::detail::lvref<const boost::system::error_code>(), \
             boost::asio::detail::lvref<const int>()), \
@@ -434,7 +431,7 @@ struct handler_type_requirements
           boost::asio::detail::rvref< \
             asio_true_handler_type>())) + \
       sizeof( \
-        boost::asio::detail::lvref< \
+        boost::asio::detail::rorlvref< \
           asio_true_handler_type>()( \
             boost::asio::detail::lvref<const boost::system::error_code>()), \
         char(0))> BOOST_ASIO_UNUSED_TYPEDEF
@@ -460,7 +457,7 @@ struct handler_type_requirements
           boost::asio::detail::rvref< \
             asio_true_handler_type>())) + \
       sizeof( \
-        boost::asio::detail::lvref< \
+        boost::asio::detail::rorlvref< \
           asio_true_handler_type>()( \
           boost::asio::detail::lvref<const boost::system::error_code>(), \
           boost::asio::detail::lvref<const std::size_t>()), \
@@ -486,7 +483,7 @@ struct handler_type_requirements
           boost::asio::detail::rvref< \
             asio_true_handler_type>())) + \
       sizeof( \
-        boost::asio::detail::lvref< \
+        boost::asio::detail::rorlvref< \
           asio_true_handler_type>()( \
             boost::asio::detail::lvref<const boost::system::error_code>()), \
         char(0))> BOOST_ASIO_UNUSED_TYPEDEF
