@@ -2,7 +2,7 @@
 // detail/win_iocp_socket_service.hpp
 // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2021 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -27,7 +27,6 @@
 #include <boost/asio/detail/buffer_sequence_adapter.hpp>
 #include <boost/asio/detail/fenced_block.hpp>
 #include <boost/asio/detail/handler_alloc_helpers.hpp>
-#include <boost/asio/detail/handler_invoke_helpers.hpp>
 #include <boost/asio/detail/memory.hpp>
 #include <boost/asio/detail/mutex.hpp>
 #include <boost/asio/detail/operation.hpp>
@@ -52,7 +51,7 @@ namespace detail {
 
 template <typename Protocol>
 class win_iocp_socket_service :
-  public execution_context_service_base<win_iocp_socket_service<Protocol> >,
+  public execution_context_service_base<win_iocp_socket_service<Protocol>>,
   public win_iocp_socket_service_base
 {
 public:
@@ -132,7 +131,7 @@ public:
   // Constructor.
   win_iocp_socket_service(execution_context& context)
     : execution_context_service_base<
-        win_iocp_socket_service<Protocol> >(context),
+        win_iocp_socket_service<Protocol>>(context),
       win_iocp_socket_service_base(context)
   {
   }
@@ -145,7 +144,7 @@ public:
 
   // Move-construct a new socket implementation.
   void move_construct(implementation_type& impl,
-      implementation_type& other_impl) BOOST_ASIO_NOEXCEPT
+      implementation_type& other_impl) noexcept
   {
     this->base_move_construct(impl, other_impl);
 
@@ -206,6 +205,8 @@ public:
       impl.have_remote_endpoint_ = false;
       impl.remote_endpoint_ = endpoint_type();
     }
+
+    BOOST_ASIO_ERROR_LOCATION(ec);
     return ec;
   }
 
@@ -220,6 +221,8 @@ public:
       impl.have_remote_endpoint_ = native_socket.have_remote_endpoint();
       impl.remote_endpoint_ = native_socket.remote_endpoint();
     }
+
+    BOOST_ASIO_ERROR_LOCATION(ec);
     return ec;
   }
 
@@ -236,6 +239,8 @@ public:
       const endpoint_type& endpoint, boost::system::error_code& ec)
   {
     socket_ops::bind(impl.socket_, endpoint.data(), endpoint.size(), ec);
+
+    BOOST_ASIO_ERROR_LOCATION(ec);
     return ec;
   }
 
@@ -247,6 +252,8 @@ public:
     socket_ops::setsockopt(impl.socket_, impl.state_,
         option.level(impl.protocol_), option.name(impl.protocol_),
         option.data(impl.protocol_), option.size(impl.protocol_), ec);
+
+    BOOST_ASIO_ERROR_LOCATION(ec);
     return ec;
   }
 
@@ -261,6 +268,8 @@ public:
         option.data(impl.protocol_), &size, ec);
     if (!ec)
       option.resize(impl.protocol_, size);
+
+    BOOST_ASIO_ERROR_LOCATION(ec);
     return ec;
   }
 
@@ -271,7 +280,10 @@ public:
     endpoint_type endpoint;
     std::size_t addr_len = endpoint.capacity();
     if (socket_ops::getsockname(impl.socket_, endpoint.data(), &addr_len, ec))
+    {
+      BOOST_ASIO_ERROR_LOCATION(ec);
       return endpoint_type();
+    }
     endpoint.resize(addr_len);
     return endpoint;
   }
@@ -284,7 +296,10 @@ public:
     std::size_t addr_len = endpoint.capacity();
     if (socket_ops::getpeername(impl.socket_, endpoint.data(),
           &addr_len, impl.have_remote_endpoint_, ec))
+    {
+      BOOST_ASIO_ERROR_LOCATION(ec);
       return endpoint_type();
+    }
     endpoint.resize(addr_len);
     return endpoint;
   }
@@ -307,9 +322,12 @@ public:
     buffer_sequence_adapter<boost::asio::const_buffer,
         ConstBufferSequence> bufs(buffers);
 
-    return socket_ops::sync_sendto(impl.socket_, impl.state_,
-        bufs.buffers(), bufs.count(), flags,
+    size_t n = socket_ops::sync_sendto(impl.socket_,
+        impl.state_, bufs.buffers(), bufs.count(), flags,
         destination.data(), destination.size(), ec);
+
+    BOOST_ASIO_ERROR_LOCATION(ec);
+    return n;
   }
 
   // Wait until data can be sent without blocking.
@@ -319,7 +337,7 @@ public:
   {
     // Wait for socket to become ready.
     socket_ops::poll_write(impl.socket_, impl.state_, -1, ec);
-
+    BOOST_ASIO_ERROR_LOCATION(ec);
     return 0;
   }
 
@@ -331,7 +349,7 @@ public:
       socket_base::message_flags flags, Handler& handler,
       const IoExecutor& io_ex)
   {
-    typename associated_cancellation_slot<Handler>::type slot
+    associated_cancellation_slot_t<Handler> slot
       = boost::asio::get_associated_cancellation_slot(handler);
 
     // Allocate and construct an operation to wrap the handler.
@@ -389,14 +407,15 @@ public:
         MutableBufferSequence> bufs(buffers);
 
     std::size_t addr_len = sender_endpoint.capacity();
-    std::size_t bytes_recvd = socket_ops::sync_recvfrom(
-        impl.socket_, impl.state_, bufs.buffers(), bufs.count(),
-        flags, sender_endpoint.data(), &addr_len, ec);
+    std::size_t n = socket_ops::sync_recvfrom(impl.socket_,
+        impl.state_, bufs.buffers(), bufs.count(), flags,
+        sender_endpoint.data(), &addr_len, ec);
 
     if (!ec)
       sender_endpoint.resize(addr_len);
 
-    return bytes_recvd;
+    BOOST_ASIO_ERROR_LOCATION(ec);
+    return n;
   }
 
   // Wait until data can be received without blocking.
@@ -410,6 +429,7 @@ public:
     // Reset endpoint since it can be given no sensible value at this time.
     sender_endpoint = endpoint_type();
 
+    BOOST_ASIO_ERROR_LOCATION(ec);
     return 0;
   }
 
@@ -423,7 +443,7 @@ public:
       socket_base::message_flags flags, Handler& handler,
       const IoExecutor& io_ex)
   {
-    typename associated_cancellation_slot<Handler>::type slot
+    associated_cancellation_slot_t<Handler> slot
       = boost::asio::get_associated_cancellation_slot(handler);
 
     // Allocate and construct an operation to wrap the handler.
@@ -455,7 +475,7 @@ public:
       endpoint_type& sender_endpoint, socket_base::message_flags flags,
       Handler& handler, const IoExecutor& io_ex)
   {
-    typename associated_cancellation_slot<Handler>::type slot
+    associated_cancellation_slot_t<Handler> slot
       = boost::asio::get_associated_cancellation_slot(handler);
 
     // Allocate and construct an operation to wrap the handler.
@@ -499,6 +519,7 @@ public:
     if (peer.is_open())
     {
       ec = boost::asio::error::already_open;
+      BOOST_ASIO_ERROR_LOCATION(ec);
       return ec;
     }
 
@@ -517,6 +538,7 @@ public:
         new_socket.release();
     }
 
+    BOOST_ASIO_ERROR_LOCATION(ec);
     return ec;
   }
 
@@ -526,7 +548,7 @@ public:
   void async_accept(implementation_type& impl, Socket& peer,
       endpoint_type* peer_endpoint, Handler& handler, const IoExecutor& io_ex)
   {
-    typename associated_cancellation_slot<Handler>::type slot
+    associated_cancellation_slot_t<Handler> slot
       = boost::asio::get_associated_cancellation_slot(handler);
 
     // Allocate and construct an operation to wrap the handler.
@@ -558,7 +580,6 @@ public:
     p.v = p.p = 0;
   }
 
-#if defined(BOOST_ASIO_HAS_MOVE)
   // Start an asynchronous accept. The peer and peer_endpoint objects
   // must be valid until the accept's handler is invoked.
   template <typename PeerIoExecutor, typename Handler, typename IoExecutor>
@@ -566,7 +587,7 @@ public:
       const PeerIoExecutor& peer_io_ex, endpoint_type* peer_endpoint,
       Handler& handler, const IoExecutor& io_ex)
   {
-    typename associated_cancellation_slot<Handler>::type slot
+    associated_cancellation_slot_t<Handler> slot
       = boost::asio::get_associated_cancellation_slot(handler);
 
     // Allocate and construct an operation to wrap the handler.
@@ -598,7 +619,6 @@ public:
         p.p->address_length(), o);
     p.v = p.p = 0;
   }
-#endif // defined(BOOST_ASIO_HAS_MOVE)
 
   // Connect the socket to the specified endpoint.
   boost::system::error_code connect(implementation_type& impl,
@@ -606,6 +626,7 @@ public:
   {
     socket_ops::sync_connect(impl.socket_,
         peer_endpoint.data(), peer_endpoint.size(), ec);
+    BOOST_ASIO_ERROR_LOCATION(ec);
     return ec;
   }
 
@@ -615,7 +636,7 @@ public:
       const endpoint_type& peer_endpoint, Handler& handler,
       const IoExecutor& io_ex)
   {
-    typename associated_cancellation_slot<Handler>::type slot
+    associated_cancellation_slot_t<Handler> slot
       = boost::asio::get_associated_cancellation_slot(handler);
 
     // Allocate and construct an operation to wrap the handler.
