@@ -2,7 +2,7 @@
 // experimental/detail/impl/channel_service.hpp
 // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2022 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -156,7 +156,7 @@ inline void channel_service<Mutex>::base_destroy(
 template <typename Mutex>
 inline std::size_t channel_service<Mutex>::capacity(
     const channel_service<Mutex>::base_implementation_type& impl)
-  const BOOST_ASIO_NOEXCEPT
+  const noexcept
 {
   typename Mutex::scoped_lock lock(impl.mutex_);
 
@@ -166,7 +166,7 @@ inline std::size_t channel_service<Mutex>::capacity(
 template <typename Mutex>
 inline bool channel_service<Mutex>::is_open(
     const channel_service<Mutex>::base_implementation_type& impl)
-  const BOOST_ASIO_NOEXCEPT
+  const noexcept
 {
   typename Mutex::scoped_lock lock(impl.mutex_);
 
@@ -205,14 +205,15 @@ void channel_service<Mutex>::close(
     {
       impl.waiters_.pop();
       traits_type::invoke_receive_closed(
-          complete_receive<payload_type,
+          post_receive<payload_type,
             typename traits_type::receive_closed_signature>(
               static_cast<channel_receive<payload_type>*>(op)));
     }
   }
 
   impl.send_state_ = closed;
-  impl.receive_state_ = closed;
+  if (impl.receive_state_ != buffer)
+    impl.receive_state_ = closed;
 }
 
 template <typename Mutex>
@@ -238,7 +239,7 @@ void channel_service<Mutex>::cancel(
     {
       impl.waiters_.pop();
       traits_type::invoke_receive_cancelled(
-          complete_receive<payload_type,
+          post_receive<payload_type,
             typename traits_type::receive_cancelled_signature>(
               static_cast<channel_receive<payload_type>*>(op)));
     }
@@ -247,7 +248,7 @@ void channel_service<Mutex>::cancel(
   if (impl.receive_state_ == waiter)
     impl.receive_state_ = block;
   if (impl.send_state_ == waiter)
-    impl.send_state_ = block;
+    impl.send_state_ = impl.max_buffer_size_ ? buffer : block;
 }
 
 template <typename Mutex>
@@ -277,7 +278,7 @@ void channel_service<Mutex>::cancel_by_key(
       {
         impl.waiters_.pop();
         traits_type::invoke_receive_cancelled(
-            complete_receive<payload_type,
+            post_receive<payload_type,
               typename traits_type::receive_cancelled_signature>(
                 static_cast<channel_receive<payload_type>*>(op)));
       }
@@ -295,14 +296,14 @@ void channel_service<Mutex>::cancel_by_key(
     if (impl.receive_state_ == waiter)
       impl.receive_state_ = block;
     if (impl.send_state_ == waiter)
-      impl.send_state_ = block;
+      impl.send_state_ = impl.max_buffer_size_ ? buffer : block;
   }
 }
 
 template <typename Mutex>
 inline bool channel_service<Mutex>::ready(
     const channel_service<Mutex>::base_implementation_type& impl)
-  const BOOST_ASIO_NOEXCEPT
+  const noexcept
 {
   typename Mutex::scoped_lock lock(impl.mutex_);
 
@@ -314,7 +315,7 @@ template <typename Message, typename Traits,
     typename... Signatures, typename... Args>
 bool channel_service<Mutex>::try_send(
     channel_service<Mutex>::implementation_type<Traits, Signatures...>& impl,
-    BOOST_ASIO_MOVE_ARG(Args)... args)
+    bool via_dispatch, Args&&... args)
 {
   typedef typename implementation_type<Traits,
       Signatures...>::payload_type payload_type;
@@ -329,7 +330,7 @@ bool channel_service<Mutex>::try_send(
     }
   case buffer:
     {
-      impl.buffer_push(Message(0, BOOST_ASIO_MOVE_CAST(Args)(args)...));
+      impl.buffer_push(Message(0, static_cast<Args&&>(args)...));
       impl.receive_state_ = buffer;
       if (impl.buffer_size() == impl.max_buffer_size_)
         impl.send_state_ = block;
@@ -337,13 +338,17 @@ bool channel_service<Mutex>::try_send(
     }
   case waiter:
     {
-      payload_type payload(Message(0, BOOST_ASIO_MOVE_CAST(Args)(args)...));
+      payload_type payload(Message(0, static_cast<Args&&>(args)...));
       channel_receive<payload_type>* receive_op =
         static_cast<channel_receive<payload_type>*>(impl.waiters_.front());
       impl.waiters_.pop();
-      receive_op->complete(BOOST_ASIO_MOVE_CAST(payload_type)(payload));
       if (impl.waiters_.empty())
         impl.send_state_ = impl.max_buffer_size_ ? buffer : block;
+      lock.unlock();
+      if (via_dispatch)
+        receive_op->dispatch(static_cast<payload_type&&>(payload));
+      else
+        receive_op->post(static_cast<payload_type&&>(payload));
       return true;
     }
   case closed:
@@ -359,7 +364,7 @@ template <typename Message, typename Traits,
     typename... Signatures, typename... Args>
 std::size_t channel_service<Mutex>::try_send_n(
     channel_service<Mutex>::implementation_type<Traits, Signatures...>& impl,
-		std::size_t count, BOOST_ASIO_MOVE_ARG(Args)... args)
+		std::size_t count, bool via_dispatch, Args&&... args)
 {
   typedef typename implementation_type<Traits,
       Signatures...>::payload_type payload_type;
@@ -381,7 +386,7 @@ std::size_t channel_service<Mutex>::try_send_n(
     return 0;
   }
 
-  payload_type payload(Message(0, BOOST_ASIO_MOVE_CAST(Args)(args)...));
+  payload_type payload(Message(0, static_cast<Args&&>(args)...));
 
   for (std::size_t i = 0; i < count; ++i)
   {
@@ -394,7 +399,7 @@ std::size_t channel_service<Mutex>::try_send_n(
     case buffer:
       {
         i += impl.buffer_push_n(count - i,
-            BOOST_ASIO_MOVE_CAST(payload_type)(payload));
+            static_cast<payload_type&&>(payload));
         impl.receive_state_ = buffer;
         if (impl.buffer_size() == impl.max_buffer_size_)
           impl.send_state_ = block;
@@ -405,9 +410,13 @@ std::size_t channel_service<Mutex>::try_send_n(
         channel_receive<payload_type>* receive_op =
           static_cast<channel_receive<payload_type>*>(impl.waiters_.front());
         impl.waiters_.pop();
-        receive_op->complete(payload);
         if (impl.waiters_.empty())
           impl.send_state_ = impl.max_buffer_size_ ? buffer : block;
+        lock.unlock();
+        if (via_dispatch)
+          receive_op->dispatch(payload);
+        else
+          receive_op->post(payload);
         break;
       }
     case closed:
@@ -448,7 +457,7 @@ void channel_service<Mutex>::start_send_op(
       impl.receive_state_ = buffer;
       if (impl.buffer_size() == impl.max_buffer_size_)
         impl.send_state_ = block;
-      send_op->complete();
+      send_op->immediate();
       break;
     }
   case waiter:
@@ -456,10 +465,10 @@ void channel_service<Mutex>::start_send_op(
       channel_receive<payload_type>* receive_op =
         static_cast<channel_receive<payload_type>*>(impl.waiters_.front());
       impl.waiters_.pop();
-      receive_op->complete(send_op->get_payload());
       if (impl.waiters_.empty())
         impl.send_state_ = impl.max_buffer_size_ ? buffer : block;
-      send_op->complete();
+      receive_op->post(send_op->get_payload());
+      send_op->immediate();
       break;
     }
   case closed:
@@ -475,7 +484,7 @@ template <typename Mutex>
 template <typename Traits, typename... Signatures, typename Handler>
 bool channel_service<Mutex>::try_receive(
     channel_service<Mutex>::implementation_type<Traits, Signatures...>& impl,
-		BOOST_ASIO_MOVE_ARG(Handler) handler)
+		Handler&& handler)
 {
   typedef typename implementation_type<Traits,
       Signatures...>::payload_type payload_type;
@@ -497,7 +506,7 @@ bool channel_service<Mutex>::try_receive(
         impl.buffer_pop();
         impl.buffer_push(send_op->get_payload());
         impl.waiters_.pop();
-        send_op->complete();
+        send_op->post();
       }
       else
       {
@@ -508,8 +517,8 @@ bool channel_service<Mutex>::try_receive(
       }
       lock.unlock();
       boost::asio::detail::non_const_lvalue<Handler> handler2(handler);
-      channel_handler<payload_type, typename decay<Handler>::type>(
-          BOOST_ASIO_MOVE_CAST(payload_type)(payload), handler2.value)();
+      channel_handler<payload_type, decay_t<Handler>>(
+          static_cast<payload_type&&>(payload), handler2.value)();
       return true;
     }
   case waiter:
@@ -518,13 +527,13 @@ bool channel_service<Mutex>::try_receive(
         static_cast<channel_send<payload_type>*>(impl.waiters_.front());
       payload_type payload = send_op->get_payload();
       impl.waiters_.pop();
-      send_op->complete();
       if (impl.waiters_.front() == 0)
         impl.receive_state_ = (impl.send_state_ == closed) ? closed : block;
+      send_op->post();
       lock.unlock();
       boost::asio::detail::non_const_lvalue<Handler> handler2(handler);
-      channel_handler<payload_type, typename decay<Handler>::type>(
-          BOOST_ASIO_MOVE_CAST(payload_type)(payload), handler2.value)();
+      channel_handler<payload_type, decay_t<Handler>>(
+          static_cast<payload_type&&>(payload), handler2.value)();
       return true;
     }
   case closed:
@@ -560,14 +569,15 @@ void channel_service<Mutex>::start_receive_op(
     }
   case buffer:
     {
-      receive_op->complete(impl.buffer_front());
+      payload_type payload(
+          static_cast<payload_type&&>(impl.buffer_front()));
       if (channel_send<payload_type>* send_op =
           static_cast<channel_send<payload_type>*>(impl.waiters_.front()))
       {
         impl.buffer_pop();
         impl.buffer_push(send_op->get_payload());
         impl.waiters_.pop();
-        send_op->complete();
+        send_op->post();
       }
       else
       {
@@ -576,6 +586,7 @@ void channel_service<Mutex>::start_receive_op(
           impl.receive_state_ = (impl.send_state_ == closed) ? closed : block;
         impl.send_state_ = (impl.send_state_ == closed) ? closed : buffer;
       }
+      receive_op->immediate(static_cast<payload_type&&>(payload));
       break;
     }
   case waiter:
@@ -584,17 +595,17 @@ void channel_service<Mutex>::start_receive_op(
         static_cast<channel_send<payload_type>*>(impl.waiters_.front());
       payload_type payload = send_op->get_payload();
       impl.waiters_.pop();
-      send_op->complete();
-      receive_op->complete(BOOST_ASIO_MOVE_CAST(payload_type)(payload));
       if (impl.waiters_.front() == 0)
         impl.receive_state_ = (impl.send_state_ == closed) ? closed : block;
+      send_op->post();
+      receive_op->immediate(static_cast<payload_type&&>(payload));
       break;
     }
   case closed:
   default:
     {
       traits_type::invoke_receive_closed(
-          complete_receive<payload_type,
+          post_receive<payload_type,
             typename traits_type::receive_closed_signature>(receive_op));
       break;
     }
