@@ -2,7 +2,7 @@
 // impl/buffered_write_stream.hpp
 // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2022 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -16,9 +16,7 @@
 #endif // defined(_MSC_VER) && (_MSC_VER >= 1200)
 
 #include <boost/asio/associator.hpp>
-#include <boost/asio/detail/handler_alloc_helpers.hpp>
 #include <boost/asio/detail/handler_cont_helpers.hpp>
-#include <boost/asio/detail/handler_invoke_helpers.hpp>
 #include <boost/asio/detail/handler_type_requirements.hpp>
 #include <boost/asio/detail/non_const_lvalue.hpp>
 
@@ -55,11 +53,10 @@ namespace detail
     buffered_flush_handler(detail::buffered_stream_storage& storage,
         WriteHandler& handler)
       : storage_(storage),
-        handler_(BOOST_ASIO_MOVE_CAST(WriteHandler)(handler))
+        handler_(static_cast<WriteHandler&&>(handler))
     {
     }
 
-#if defined(BOOST_ASIO_HAS_MOVE)
     buffered_flush_handler(const buffered_flush_handler& other)
       : storage_(other.storage_),
         handler_(other.handler_)
@@ -68,16 +65,15 @@ namespace detail
 
     buffered_flush_handler(buffered_flush_handler&& other)
       : storage_(other.storage_),
-        handler_(BOOST_ASIO_MOVE_CAST(WriteHandler)(other.handler_))
+        handler_(static_cast<WriteHandler&&>(other.handler_))
     {
     }
-#endif // defined(BOOST_ASIO_HAS_MOVE)
 
     void operator()(const boost::system::error_code& ec,
         const std::size_t bytes_written)
     {
       storage_.consume(bytes_written);
-      BOOST_ASIO_MOVE_OR_LVALUE(WriteHandler)(handler_)(ec, bytes_written);
+      static_cast<WriteHandler&&>(handler_)(ec, bytes_written);
     }
 
   //private:
@@ -85,32 +81,6 @@ namespace detail
     WriteHandler handler_;
   };
 
-  template <typename WriteHandler>
-  inline asio_handler_allocate_is_deprecated
-  asio_handler_allocate(std::size_t size,
-      buffered_flush_handler<WriteHandler>* this_handler)
-  {
-#if defined(BOOST_ASIO_NO_DEPRECATED)
-    boost_asio_handler_alloc_helpers::allocate(size, this_handler->handler_);
-    return asio_handler_allocate_is_no_longer_used();
-#else // defined(BOOST_ASIO_NO_DEPRECATED)
-    return boost_asio_handler_alloc_helpers::allocate(
-        size, this_handler->handler_);
-#endif // defined(BOOST_ASIO_NO_DEPRECATED)
-  }
-
-  template <typename WriteHandler>
-  inline asio_handler_deallocate_is_deprecated
-  asio_handler_deallocate(void* pointer, std::size_t size,
-      buffered_flush_handler<WriteHandler>* this_handler)
-  {
-    boost_asio_handler_alloc_helpers::deallocate(
-        pointer, size, this_handler->handler_);
-#if defined(BOOST_ASIO_NO_DEPRECATED)
-    return asio_handler_deallocate_is_no_longer_used();
-#endif // defined(BOOST_ASIO_NO_DEPRECATED)
-  }
-
   template <typename WriteHandler>
   inline bool asio_handler_is_continuation(
       buffered_flush_handler<WriteHandler>* this_handler)
@@ -119,50 +89,26 @@ namespace detail
           this_handler->handler_);
   }
 
-  template <typename Function, typename WriteHandler>
-  inline asio_handler_invoke_is_deprecated
-  asio_handler_invoke(Function& function,
-      buffered_flush_handler<WriteHandler>* this_handler)
-  {
-    boost_asio_handler_invoke_helpers::invoke(
-        function, this_handler->handler_);
-#if defined(BOOST_ASIO_NO_DEPRECATED)
-    return asio_handler_invoke_is_no_longer_used();
-#endif // defined(BOOST_ASIO_NO_DEPRECATED)
-  }
-
-  template <typename Function, typename WriteHandler>
-  inline asio_handler_invoke_is_deprecated
-  asio_handler_invoke(const Function& function,
-      buffered_flush_handler<WriteHandler>* this_handler)
-  {
-    boost_asio_handler_invoke_helpers::invoke(
-        function, this_handler->handler_);
-#if defined(BOOST_ASIO_NO_DEPRECATED)
-    return asio_handler_invoke_is_no_longer_used();
-#endif // defined(BOOST_ASIO_NO_DEPRECATED)
-  }
-
   template <typename Stream>
   class initiate_async_buffered_flush
   {
   public:
-    typedef typename remove_reference<
-      Stream>::type::lowest_layer_type::executor_type executor_type;
+    typedef typename remove_reference_t<
+      Stream>::lowest_layer_type::executor_type executor_type;
 
     explicit initiate_async_buffered_flush(
-        typename remove_reference<Stream>::type& next_layer)
+        remove_reference_t<Stream>& next_layer)
       : next_layer_(next_layer)
     {
     }
 
-    executor_type get_executor() const BOOST_ASIO_NOEXCEPT
+    executor_type get_executor() const noexcept
     {
       return next_layer_.lowest_layer().get_executor();
     }
 
     template <typename WriteHandler>
-    void operator()(BOOST_ASIO_MOVE_ARG(WriteHandler) handler,
+    void operator()(WriteHandler&& handler,
         buffered_stream_storage* storage) const
     {
       // If you get an error on the following line it means that your handler
@@ -171,12 +117,12 @@ namespace detail
 
       non_const_lvalue<WriteHandler> handler2(handler);
       async_write(next_layer_, buffer(storage->data(), storage->size()),
-          buffered_flush_handler<typename decay<WriteHandler>::type>(
+          buffered_flush_handler<decay_t<WriteHandler>>(
             *storage, handler2.value));
     }
 
   private:
-    typename remove_reference<Stream>::type& next_layer_;
+    remove_reference_t<Stream>& next_layer_;
   };
 } // namespace detail
 
@@ -190,8 +136,14 @@ struct associator<Associator,
   : Associator<WriteHandler, DefaultCandidate>
 {
   static typename Associator<WriteHandler, DefaultCandidate>::type get(
-      const detail::buffered_flush_handler<WriteHandler>& h,
-      const DefaultCandidate& c = DefaultCandidate()) BOOST_ASIO_NOEXCEPT
+      const detail::buffered_flush_handler<WriteHandler>& h) noexcept
+  {
+    return Associator<WriteHandler, DefaultCandidate>::get(h.handler_);
+  }
+
+  static auto get(const detail::buffered_flush_handler<WriteHandler>& h,
+      const DefaultCandidate& c) noexcept
+    -> decltype(Associator<WriteHandler, DefaultCandidate>::get(h.handler_, c))
   {
     return Associator<WriteHandler, DefaultCandidate>::get(h.handler_, c);
   }
@@ -203,15 +155,12 @@ template <typename Stream>
 template <
     BOOST_ASIO_COMPLETION_TOKEN_FOR(void (boost::system::error_code,
       std::size_t)) WriteHandler>
-BOOST_ASIO_INITFN_AUTO_RESULT_TYPE_PREFIX(WriteHandler,
-    void (boost::system::error_code, std::size_t))
-buffered_write_stream<Stream>::async_flush(
-    BOOST_ASIO_MOVE_ARG(WriteHandler) handler)
-  BOOST_ASIO_INITFN_AUTO_RESULT_TYPE_SUFFIX((
+inline auto buffered_write_stream<Stream>::async_flush(WriteHandler&& handler)
+  -> decltype(
     async_initiate<WriteHandler,
       void (boost::system::error_code, std::size_t)>(
-        declval<detail::initiate_async_buffered_flush<Stream> >(),
-        handler, declval<detail::buffered_stream_storage*>())))
+        declval<detail::initiate_async_buffered_flush<Stream>>(),
+        handler, declval<detail::buffered_stream_storage*>()))
 {
   return async_initiate<WriteHandler,
     void (boost::system::error_code, std::size_t)>(
@@ -261,32 +210,30 @@ namespace detail
         const ConstBufferSequence& buffers, WriteHandler& handler)
       : storage_(storage),
         buffers_(buffers),
-        handler_(BOOST_ASIO_MOVE_CAST(WriteHandler)(handler))
+        handler_(static_cast<WriteHandler&&>(handler))
     {
     }
 
-#if defined(BOOST_ASIO_HAS_MOVE)
-      buffered_write_some_handler(const buffered_write_some_handler& other)
-        : storage_(other.storage_),
-          buffers_(other.buffers_),
-          handler_(other.handler_)
-      {
-      }
+    buffered_write_some_handler(const buffered_write_some_handler& other)
+      : storage_(other.storage_),
+        buffers_(other.buffers_),
+        handler_(other.handler_)
+    {
+    }
 
-      buffered_write_some_handler(buffered_write_some_handler&& other)
-        : storage_(other.storage_),
-          buffers_(other.buffers_),
-          handler_(BOOST_ASIO_MOVE_CAST(WriteHandler)(other.handler_))
-      {
-      }
-#endif // defined(BOOST_ASIO_HAS_MOVE)
+    buffered_write_some_handler(buffered_write_some_handler&& other)
+      : storage_(other.storage_),
+        buffers_(other.buffers_),
+        handler_(static_cast<WriteHandler&&>(other.handler_))
+    {
+    }
 
     void operator()(const boost::system::error_code& ec, std::size_t)
     {
       if (ec)
       {
         const std::size_t length = 0;
-        BOOST_ASIO_MOVE_OR_LVALUE(WriteHandler)(handler_)(ec, length);
+        static_cast<WriteHandler&&>(handler_)(ec, length);
       }
       else
       {
@@ -299,7 +246,7 @@ namespace detail
         storage_.resize(orig_size + length);
         const std::size_t bytes_copied = boost::asio::buffer_copy(
             storage_.data() + orig_size, buffers_, length);
-        BOOST_ASIO_MOVE_OR_LVALUE(WriteHandler)(handler_)(ec, bytes_copied);
+        static_cast<WriteHandler&&>(handler_)(ec, bytes_copied);
       }
     }
 
@@ -309,34 +256,6 @@ namespace detail
     WriteHandler handler_;
   };
 
-  template <typename ConstBufferSequence, typename WriteHandler>
-  inline asio_handler_allocate_is_deprecated
-  asio_handler_allocate(std::size_t size,
-      buffered_write_some_handler<
-        ConstBufferSequence, WriteHandler>* this_handler)
-  {
-#if defined(BOOST_ASIO_NO_DEPRECATED)
-    boost_asio_handler_alloc_helpers::allocate(size, this_handler->handler_);
-    return asio_handler_allocate_is_no_longer_used();
-#else // defined(BOOST_ASIO_NO_DEPRECATED)
-    return boost_asio_handler_alloc_helpers::allocate(
-        size, this_handler->handler_);
-#endif // defined(BOOST_ASIO_NO_DEPRECATED)
-  }
-
-  template <typename ConstBufferSequence, typename WriteHandler>
-  inline asio_handler_deallocate_is_deprecated
-  asio_handler_deallocate(void* pointer, std::size_t size,
-      buffered_write_some_handler<
-        ConstBufferSequence, WriteHandler>* this_handler)
-  {
-    boost_asio_handler_alloc_helpers::deallocate(
-        pointer, size, this_handler->handler_);
-#if defined(BOOST_ASIO_NO_DEPRECATED)
-    return asio_handler_deallocate_is_no_longer_used();
-#endif // defined(BOOST_ASIO_NO_DEPRECATED)
-  }
-
   template <typename ConstBufferSequence, typename WriteHandler>
   inline bool asio_handler_is_continuation(
       buffered_write_some_handler<
@@ -346,54 +265,26 @@ namespace detail
           this_handler->handler_);
   }
 
-  template <typename Function, typename ConstBufferSequence,
-      typename WriteHandler>
-  inline asio_handler_invoke_is_deprecated
-  asio_handler_invoke(Function& function,
-      buffered_write_some_handler<
-        ConstBufferSequence, WriteHandler>* this_handler)
-  {
-    boost_asio_handler_invoke_helpers::invoke(
-        function, this_handler->handler_);
-#if defined(BOOST_ASIO_NO_DEPRECATED)
-    return asio_handler_invoke_is_no_longer_used();
-#endif // defined(BOOST_ASIO_NO_DEPRECATED)
-  }
-
-  template <typename Function, typename ConstBufferSequence,
-      typename WriteHandler>
-  inline asio_handler_invoke_is_deprecated
-  asio_handler_invoke(const Function& function,
-      buffered_write_some_handler<
-        ConstBufferSequence, WriteHandler>* this_handler)
-  {
-    boost_asio_handler_invoke_helpers::invoke(
-        function, this_handler->handler_);
-#if defined(BOOST_ASIO_NO_DEPRECATED)
-    return asio_handler_invoke_is_no_longer_used();
-#endif // defined(BOOST_ASIO_NO_DEPRECATED)
-  }
-
   template <typename Stream>
   class initiate_async_buffered_write_some
   {
   public:
-    typedef typename remove_reference<
-      Stream>::type::lowest_layer_type::executor_type executor_type;
+    typedef typename remove_reference_t<
+      Stream>::lowest_layer_type::executor_type executor_type;
 
     explicit initiate_async_buffered_write_some(
-        typename remove_reference<Stream>::type& next_layer)
+        remove_reference_t<Stream>& next_layer)
       : next_layer_(next_layer)
     {
     }
 
-    executor_type get_executor() const BOOST_ASIO_NOEXCEPT
+    executor_type get_executor() const noexcept
     {
       return next_layer_.lowest_layer().get_executor();
     }
 
     template <typename WriteHandler, typename ConstBufferSequence>
-    void operator()(BOOST_ASIO_MOVE_ARG(WriteHandler) handler,
+    void operator()(WriteHandler&& handler,
         buffered_stream_storage* storage,
         const ConstBufferSequence& buffers) const
     {
@@ -407,21 +298,21 @@ namespace detail
       {
         next_layer_.async_write_some(BOOST_ASIO_CONST_BUFFER(0, 0),
             buffered_write_some_handler<ConstBufferSequence,
-              typename decay<WriteHandler>::type>(
+              decay_t<WriteHandler>>(
                 *storage, buffers, handler2.value));
       }
       else
       {
         initiate_async_buffered_flush<Stream>(this->next_layer_)(
             buffered_write_some_handler<ConstBufferSequence,
-              typename decay<WriteHandler>::type>(
+              decay_t<WriteHandler>>(
                 *storage, buffers, handler2.value),
             storage);
       }
     }
 
   private:
-    typename remove_reference<Stream>::type& next_layer_;
+    remove_reference_t<Stream>& next_layer_;
   };
 } // namespace detail
 
@@ -436,9 +327,17 @@ struct associator<Associator,
   : Associator<WriteHandler, DefaultCandidate>
 {
   static typename Associator<WriteHandler, DefaultCandidate>::type get(
+      const detail::buffered_write_some_handler<
+        ConstBufferSequence, WriteHandler>& h) noexcept
+  {
+    return Associator<WriteHandler, DefaultCandidate>::get(h.handler_);
+  }
+
+  static auto get(
       const detail::buffered_write_some_handler<
         ConstBufferSequence, WriteHandler>& h,
-      const DefaultCandidate& c = DefaultCandidate()) BOOST_ASIO_NOEXCEPT
+      const DefaultCandidate& c) noexcept
+    -> decltype(Associator<WriteHandler, DefaultCandidate>::get(h.handler_, c))
   {
     return Associator<WriteHandler, DefaultCandidate>::get(h.handler_, c);
   }
@@ -450,16 +349,13 @@ template <typename Stream>
 template <typename ConstBufferSequence,
     BOOST_ASIO_COMPLETION_TOKEN_FOR(void (boost::system::error_code,
       std::size_t)) WriteHandler>
-BOOST_ASIO_INITFN_AUTO_RESULT_TYPE_PREFIX(WriteHandler,
-    void (boost::system::error_code, std::size_t))
-buffered_write_stream<Stream>::async_write_some(
-    const ConstBufferSequence& buffers,
-    BOOST_ASIO_MOVE_ARG(WriteHandler) handler)
-  BOOST_ASIO_INITFN_AUTO_RESULT_TYPE_SUFFIX((
+inline auto buffered_write_stream<Stream>::async_write_some(
+    const ConstBufferSequence& buffers, WriteHandler&& handler)
+  -> decltype(
     async_initiate<WriteHandler,
       void (boost::system::error_code, std::size_t)>(
-        declval<detail::initiate_async_buffered_write_some<Stream> >(),
-        handler, declval<detail::buffered_stream_storage*>(), buffers)))
+        declval<detail::initiate_async_buffered_write_some<Stream>>(),
+        handler, declval<detail::buffered_stream_storage*>(), buffers))
 {
   return async_initiate<WriteHandler,
     void (boost::system::error_code, std::size_t)>(
