@@ -2,7 +2,7 @@
 // impl/src.hpp
 // ~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2022 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -19,13 +19,13 @@
 # error Do not compile Asio library source with BOOST_ASIO_HEADER_ONLY defined
 #endif
 
+#include <boost/asio/impl/any_completion_executor.ipp>
 #include <boost/asio/impl/any_io_executor.ipp>
 #include <boost/asio/impl/cancellation_signal.ipp>
 #include <boost/asio/impl/connect_pipe.ipp>
 #include <boost/asio/impl/error.ipp>
 #include <boost/asio/impl/execution_context.ipp>
 #include <boost/asio/impl/executor.ipp>
-#include <boost/asio/impl/handler_alloc_hook.ipp>
 #include <boost/asio/impl/io_context.ipp>
 #include <boost/asio/impl/multiple_exceptions.ipp>
 #include <boost/asio/impl/serial_port_base.ipp>
@@ -79,7 +79,6 @@
 #include <boost/asio/detail/impl/winrt_timer_scheduler.ipp>
 #include <boost/asio/detail/impl/winsock_init.ipp>
 #include <boost/asio/execution/impl/bad_executor.ipp>
-#include <boost/asio/execution/impl/receiver_invocation_error.ipp>
 #include <boost/asio/experimental/impl/channel_error.ipp>
 #include <boost/asio/generic/detail/impl/endpoint.ipp>
 #include <boost/asio/ip/impl/address.ipp>
