@@ -1,9 +1,10 @@
 // Boost.Geometry (aka GGL, Generic Geometry Library)
 
 // Copyright (c) 2012-2020 Barend Gehrels, Amsterdam, the Netherlands.
+// Copyright (c) 2022-2023 Adam Wulkiewicz, Lodz, Poland.
 
-// This file was modified by Oracle on 2017-2021.
-// Modifications copyright (c) 2017-2021 Oracle and/or its affiliates.
+// This file was modified by Oracle on 2017-2022.
+// Modifications copyright (c) 2017-2022 Oracle and/or its affiliates.
 // Contributed and/or modified by Adam Wulkiewicz, on behalf of Oracle
 
 // Use, modification and distribution is subject to the Boost Software License,
@@ -37,14 +38,17 @@
 #include <boost/geometry/core/exterior_ring.hpp>
 #include <boost/geometry/core/interior_rings.hpp>
 
+#include <boost/geometry/geometries/linestring.hpp>
+#include <boost/geometry/geometries/ring.hpp>
+
 #include <boost/geometry/strategies/buffer.hpp>
 #include <boost/geometry/strategies/side.hpp>
 
-#include <boost/geometry/util/condition.hpp>
+#include <boost/geometry/util/constexpr.hpp>
 #include <boost/geometry/util/math.hpp>
 #include <boost/geometry/util/type_traits.hpp>
 
-#include <boost/geometry/views/detail/normalized_view.hpp>
+#include <boost/geometry/views/detail/closed_clockwise_view.hpp>
 
 
 namespace boost { namespace geometry
@@ -54,10 +58,11 @@ namespace boost { namespace geometry
 namespace detail { namespace buffer
 {
 
-template <typename Range, typename DistanceStrategy>
-inline void simplify_input(Range const& range,
-        DistanceStrategy const& distance,
-        Range& simplified)
+template <typename RangeIn, typename DistanceStrategy, typename RangeOut, typename Strategies>
+inline void simplify_input(RangeIn const& range,
+                           DistanceStrategy const& distance,
+                           RangeOut& simplified,
+                           Strategies const& strategies)
 {
     // We have to simplify the ring before to avoid very small-scaled
     // features in the original (convex/concave/convex) being enlarged
@@ -69,19 +74,10 @@ inline void simplify_input(Range const& range,
     // look better.
     // It also gets rid of duplicate points
 
-    typedef typename geometry::point_type<Range>::type point_type;
-    typedef typename strategy::distance::services::default_strategy
-    <
-        point_tag, segment_tag, point_type
-    >::type ds_strategy_type;
-    typedef strategy::simplify::douglas_peucker
-    <
-        point_type, ds_strategy_type
-    > strategy_type;
-
     geometry::detail::simplify::simplify_range<2>::apply(range,
         simplified, distance.simplify_distance(),
-        strategy_type());
+        detail::simplify::douglas_peucker(),
+        strategies);
 
 }
 
@@ -101,7 +97,7 @@ struct buffer_range
         typename JoinStrategy,
         typename EndStrategy,
         typename RobustPolicy,
-        typename SideStrategy
+        typename Strategies
     >
     static inline
     void add_join(Collection& collection,
@@ -118,11 +114,11 @@ struct buffer_range
             JoinStrategy const& join_strategy,
             EndStrategy const& end_strategy,
             RobustPolicy const& ,
-            SideStrategy const& side_strategy)
+            Strategies const& strategies)
     {
         geometry::strategy::buffer::join_selector const join
                 = get_join_type(penultimate_input, previous_input, input,
-                                side_strategy);
+                                strategies);
 
         switch(join)
         {
@@ -187,14 +183,14 @@ struct buffer_range
         return direction_code<cs_tag>(p0, p1, p2) == 1;
     }
 
-    template <typename SideStrategy>
+    template <typename Strategies>
     static inline geometry::strategy::buffer::join_selector get_join_type(
             output_point_type const& p0,
             output_point_type const& p1,
             output_point_type const& p2,
-            SideStrategy const& side_strategy)
+            Strategies const& strategies)
     {
-        int const side = side_strategy.apply(p0, p1, p2);
+        int const side = strategies.side().apply(p0, p1, p2);
         return side == -1 ? geometry::strategy::buffer::join_convex
             :  side == 1  ? geometry::strategy::buffer::join_concave
             :  same_direction(p0, p1, p2) ? geometry::strategy::buffer::join_continue
@@ -210,7 +206,7 @@ struct buffer_range
         typename JoinStrategy,
         typename EndStrategy,
         typename RobustPolicy,
-        typename SideStrategy
+        typename Strategies
     >
     static inline geometry::strategy::buffer::result_code iterate(Collection& collection,
                 Iterator begin, Iterator end,
@@ -220,7 +216,7 @@ struct buffer_range
                 JoinStrategy const& join_strategy,
                 EndStrategy const& end_strategy,
                 RobustPolicy const& robust_policy,
-                SideStrategy const& side_strategy,
+                Strategies const& strategies,
                 bool linear,
                 output_point_type& first_p1,
                 output_point_type& first_p2,
@@ -296,10 +292,10 @@ struct buffer_range
                         *it, generated_side.front(), generated_side.back(),
                         side,
                         distance_strategy, segment_strategy, join_strategy, end_strategy,
-                        robust_policy, side_strategy);
+                        robust_policy, strategies);
             }
 
-            collection.add_side_piece(*prev, *it, generated_side, first);
+            collection.add_side_piece(*prev, *it, generated_side, first, distance_strategy.empty(side));
 
             if (first && mark_flat)
             {
@@ -346,7 +342,7 @@ struct buffer_multi
         typename EndStrategy,
         typename PointStrategy,
         typename RobustPolicy,
-        typename SideStrategy
+        typename Strategies
     >
     static inline void apply(Multi const& multi,
             Collection& collection,
@@ -356,17 +352,14 @@ struct buffer_multi
             EndStrategy const& end_strategy,
             PointStrategy const& point_strategy,
             RobustPolicy const& robust_policy,
-            SideStrategy const& side_strategy)
+            Strategies const& strategies)
     {
-        for (typename boost::range_iterator<Multi const>::type
-                it = boost::begin(multi);
-            it != boost::end(multi);
-            ++it)
+        for (auto it = boost::begin(multi); it != boost::end(multi); ++it)
         {
             Policy::apply(*it, collection,
                 distance_strategy, segment_strategy,
                 join_strategy, end_strategy, point_strategy,
-                robust_policy, side_strategy);
+                robust_policy, strategies);
         }
     }
 };
@@ -434,7 +427,7 @@ struct buffer_inserter<point_tag, Point, RingOutput>
         typename EndStrategy,
         typename PointStrategy,
         typename RobustPolicy,
-        typename SideStrategy
+        typename Strategies
     >
     static inline void apply(Point const& point, Collection& collection,
             DistanceStrategy const& distance_strategy,
@@ -443,7 +436,7 @@ struct buffer_inserter<point_tag, Point, RingOutput>
             EndStrategy const& ,
             PointStrategy const& point_strategy,
             RobustPolicy const& ,
-            SideStrategy const& )
+            Strategies const& )
     {
         detail::buffer::buffer_point
         <
@@ -461,7 +454,7 @@ template
 >
 struct buffer_inserter_ring
 {
-    typedef typename point_type<RingOutput>::type output_point_type;
+    using output_point_type = typename point_type<RingOutput>::type;
 
     template
     <
@@ -472,7 +465,7 @@ struct buffer_inserter_ring
         typename JoinStrategy,
         typename EndStrategy,
         typename RobustPolicy,
-        typename SideStrategy
+        typename Strategies
     >
     static inline geometry::strategy::buffer::result_code iterate(Collection& collection,
                 Iterator begin, Iterator end,
@@ -482,7 +475,7 @@ struct buffer_inserter_ring
                 JoinStrategy const& join_strategy,
                 EndStrategy const& end_strategy,
                 RobustPolicy const& robust_policy,
-                SideStrategy const& side_strategy)
+                Strategies const& strategies)
     {
         output_point_type first_p1, first_p2, last_p1, last_p2;
 
@@ -492,7 +485,7 @@ struct buffer_inserter_ring
             = buffer_range::iterate(collection, begin, end,
                 side,
                 distance_strategy, segment_strategy, join_strategy, end_strategy,
-                robust_policy, side_strategy,
+                robust_policy, strategies,
                 false, first_p1, first_p2, last_p1, last_p2);
 
         // Generate closing join
@@ -504,7 +497,7 @@ struct buffer_inserter_ring
                 *(begin + 1), first_p1, first_p2,
                 side,
                 distance_strategy, segment_strategy, join_strategy, end_strategy,
-                robust_policy, side_strategy);
+                robust_policy, strategies);
         }
 
         // Buffer is closed automatically by last closing corner
@@ -520,7 +513,7 @@ struct buffer_inserter_ring
         typename EndStrategy,
         typename PointStrategy,
         typename RobustPolicy,
-        typename SideStrategy
+        typename Strategies
     >
     static inline geometry::strategy::buffer::result_code apply(RingInput const& ring,
             Collection& collection,
@@ -530,36 +523,43 @@ struct buffer_inserter_ring
             EndStrategy const& end_strategy,
             PointStrategy const& point_strategy,
             RobustPolicy const& robust_policy,
-            SideStrategy const& side_strategy)
+            Strategies const& strategies)
     {
-        RingInput simplified;
-        detail::buffer::simplify_input(ring, distance, simplified);
+        // Use helper geometry to support non-mutable input Rings
+        using simplified_ring_t = model::ring
+            <
+                output_point_type,
+                point_order<RingInput>::value != counterclockwise,
+                closure<RingInput>::value != open
+            >;
+        simplified_ring_t simplified;
+        detail::buffer::simplify_input(ring, distance, simplified, strategies);
 
         geometry::strategy::buffer::result_code code = geometry::strategy::buffer::result_no_output;
 
         std::size_t n = boost::size(simplified);
         std::size_t const min_points = core_detail::closure::minimum_ring_size
             <
-                geometry::closure<RingInput>::value
+                geometry::closure<simplified_ring_t>::value
             >::value;
 
         if (n >= min_points)
         {
-            detail::normalized_view<RingInput const> view(simplified);
+            detail::closed_clockwise_view<simplified_ring_t const> view(simplified);
             if (distance.negative())
             {
                 // Walk backwards (rings will be reversed afterwards)
                 code = iterate(collection, boost::rbegin(view), boost::rend(view),
                         geometry::strategy::buffer::buffer_side_right,
                         distance, segment_strategy, join_strategy, end_strategy,
-                        robust_policy, side_strategy);
+                        robust_policy, strategies);
             }
             else
             {
                 code = iterate(collection, boost::begin(view), boost::end(view),
                         geometry::strategy::buffer::buffer_side_left,
                         distance, segment_strategy, join_strategy, end_strategy,
-                        robust_policy, side_strategy);
+                        robust_policy, strategies);
             }
         }
 
@@ -593,7 +593,7 @@ struct buffer_inserter<ring_tag, RingInput, RingOutput>
         typename EndStrategy,
         typename PointStrategy,
         typename RobustPolicy,
-        typename SideStrategy
+        typename Strategies
     >
     static inline geometry::strategy::buffer::result_code apply(RingInput const& ring,
             Collection& collection,
@@ -603,14 +603,14 @@ struct buffer_inserter<ring_tag, RingInput, RingOutput>
             EndStrategy const& end_strategy,
             PointStrategy const& point_strategy,
             RobustPolicy const& robust_policy,
-            SideStrategy const& side_strategy)
+            Strategies const& strategies)
     {
         collection.start_new_ring(distance.negative());
         geometry::strategy::buffer::result_code const code
             = buffer_inserter_ring<RingInput, RingOutput>::apply(ring,
                 collection, distance,
                 segment_strategy, join_strategy, end_strategy, point_strategy,
-                robust_policy, side_strategy);
+                robust_policy, strategies);
         collection.finish_ring(code, ring, false, false);
         return code;
     }
@@ -623,9 +623,8 @@ template
 >
 struct buffer_inserter<linestring_tag, Linestring, Polygon>
 {
-    typedef typename ring_type<Polygon>::type output_ring_type;
-    typedef typename point_type<output_ring_type>::type output_point_type;
-    typedef typename point_type<Linestring>::type input_point_type;
+    using output_ring_type = typename ring_type<Polygon>::type;
+    using output_point_type = typename point_type<output_ring_type>::type;
 
     template
     <
@@ -636,7 +635,7 @@ struct buffer_inserter<linestring_tag, Linestring, Polygon>
         typename JoinStrategy,
         typename EndStrategy,
         typename RobustPolicy,
-        typename SideStrategy
+        typename Strategies
     >
     static inline geometry::strategy::buffer::result_code iterate(Collection& collection,
                 Iterator begin, Iterator end,
@@ -646,11 +645,11 @@ struct buffer_inserter<linestring_tag, Linestring, Polygon>
                 JoinStrategy const& join_strategy,
                 EndStrategy const& end_strategy,
                 RobustPolicy const& robust_policy,
-                SideStrategy const& side_strategy,
+                Strategies const& strategies,
                 output_point_type& first_p1)
     {
-        input_point_type const& ultimate_point = *(end - 1);
-        input_point_type const& penultimate_point = *(end - 2);
+        output_point_type const& ultimate_point = *(end - 1);
+        output_point_type const& penultimate_point = *(end - 2);
 
         // For the end-cap, we need to have the last perpendicular point on the
         // other side of the linestring. If it is the second pass (right),
@@ -682,7 +681,7 @@ struct buffer_inserter<linestring_tag, Linestring, Polygon>
             = detail::buffer::buffer_range<output_ring_type>::iterate(collection,
                 begin, end, side,
                 distance_strategy, segment_strategy, join_strategy, end_strategy,
-                robust_policy, side_strategy,
+                robust_policy, strategies,
                 true, first_p1, first_p2, last_p1, last_p2);
 
         if (result == geometry::strategy::buffer::result_normal)
@@ -704,7 +703,7 @@ struct buffer_inserter<linestring_tag, Linestring, Polygon>
         typename EndStrategy,
         typename PointStrategy,
         typename RobustPolicy,
-        typename SideStrategy
+        typename Strategies
     >
     static inline geometry::strategy::buffer::result_code apply(Linestring const& linestring,
             Collection& collection,
@@ -714,10 +713,11 @@ struct buffer_inserter<linestring_tag, Linestring, Polygon>
             EndStrategy const& end_strategy,
             PointStrategy const& point_strategy,
             RobustPolicy const& robust_policy,
-            SideStrategy const& side_strategy)
+            Strategies const& strategies)
     {
-        Linestring simplified;
-        detail::buffer::simplify_input(linestring, distance, simplified);
+        // Use helper geometry to support non-mutable input Linestrings
+        model::linestring<output_point_type> simplified;
+        detail::buffer::simplify_input(linestring, distance, simplified, strategies);
 
         geometry::strategy::buffer::result_code code = geometry::strategy::buffer::result_no_output;
         std::size_t n = boost::size(simplified);
@@ -729,7 +729,7 @@ struct buffer_inserter<linestring_tag, Linestring, Polygon>
                     boost::begin(simplified), boost::end(simplified),
                     geometry::strategy::buffer::buffer_side_left,
                     distance, segment_strategy, join_strategy, end_strategy,
-                    robust_policy, side_strategy,
+                    robust_policy, strategies,
                     first_p1);
 
             if (code == geometry::strategy::buffer::result_normal)
@@ -738,7 +738,7 @@ struct buffer_inserter<linestring_tag, Linestring, Polygon>
                         boost::rbegin(simplified), boost::rend(simplified),
                         geometry::strategy::buffer::buffer_side_right,
                         distance, segment_strategy, join_strategy, end_strategy,
-                        robust_policy, side_strategy,
+                        robust_policy, strategies,
                         first_p1);
             }
             collection.finish_ring(code);
@@ -781,7 +781,7 @@ private:
         typename EndStrategy,
         typename PointStrategy,
         typename RobustPolicy,
-        typename SideStrategy
+        typename Strategies
     >
     static inline
     void iterate(Iterator begin, Iterator end,
@@ -792,7 +792,7 @@ private:
             EndStrategy const& end_strategy,
             PointStrategy const& point_strategy,
             RobustPolicy const& robust_policy,
-            SideStrategy const& side_strategy,
+            Strategies const& strategies,
             bool is_interior)
     {
         for (Iterator it = begin; it != end; ++it)
@@ -807,7 +807,7 @@ private:
             geometry::strategy::buffer::result_code const code
                     = policy::apply(*it, collection, distance, segment_strategy,
                     join_strategy, end_strategy, point_strategy,
-                    robust_policy, side_strategy);
+                    robust_policy, strategies);
 
             collection.finish_ring(code, *it, is_interior, false);
         }
@@ -823,7 +823,7 @@ private:
         typename EndStrategy,
         typename PointStrategy,
         typename RobustPolicy,
-        typename SideStrategy
+        typename Strategies
     >
     static inline
     void apply_interior_rings(InteriorRings const& interior_rings,
@@ -834,12 +834,12 @@ private:
             EndStrategy const& end_strategy,
             PointStrategy const& point_strategy,
             RobustPolicy const& robust_policy,
-            SideStrategy const& side_strategy)
+            Strategies const& strategies)
     {
         iterate(boost::begin(interior_rings), boost::end(interior_rings),
             collection, distance, segment_strategy,
             join_strategy, end_strategy, point_strategy,
-            robust_policy, side_strategy, true);
+            robust_policy, strategies, true);
     }
 
 public:
@@ -852,7 +852,7 @@ public:
         typename EndStrategy,
         typename PointStrategy,
         typename RobustPolicy,
-        typename SideStrategy
+        typename Strategies
     >
     static inline void apply(PolygonInput const& polygon,
             Collection& collection,
@@ -862,7 +862,7 @@ public:
             EndStrategy const& end_strategy,
             PointStrategy const& point_strategy,
             RobustPolicy const& robust_policy,
-            SideStrategy const& side_strategy)
+            Strategies const& strategies)
     {
         {
             collection.start_new_ring(distance.negative());
@@ -871,7 +871,7 @@ public:
                 = policy::apply(exterior_ring(polygon), collection,
                     distance, segment_strategy,
                     join_strategy, end_strategy, point_strategy,
-                    robust_policy, side_strategy);
+                    robust_policy, strategies);
 
             collection.finish_ring(code, exterior_ring(polygon), false,
                     geometry::num_interior_rings(polygon) > 0u);
@@ -880,7 +880,7 @@ public:
         apply_interior_rings(interior_rings(polygon),
                 collection, distance, segment_strategy,
                 join_strategy, end_strategy, point_strategy,
-                robust_policy, side_strategy);
+                robust_policy, strategies);
     }
 };
 
@@ -942,17 +942,17 @@ inline void buffer_inserter(GeometryInput const& geometry_input, OutputIterator
 {
     boost::ignore_unused(visit_pieces_policy);
 
-    typedef detail::buffer::buffered_piece_collection
-    <
-        typename geometry::ring_type<GeometryOutput>::type,
-        Strategies,
-        DistanceStrategy,
-        RobustPolicy
-    > collection_type;
+    using collection_type = detail::buffer::buffered_piece_collection
+        <
+            typename geometry::ring_type<GeometryOutput>::type,
+            Strategies,
+            DistanceStrategy,
+            RobustPolicy
+        >;
     collection_type collection(strategies, distance_strategy, robust_policy);
     collection_type const& const_collection = collection;
 
-    bool const areal = util::is_areal<GeometryInput>::value;
+    static constexpr bool areal = util::is_areal<GeometryInput>::value;
 
     dispatch::buffer_inserter
         <
@@ -964,12 +964,12 @@ inline void buffer_inserter(GeometryInput const& geometry_input, OutputIterator
             GeometryInput,
             GeometryOutput
         >::apply(geometry_input, collection,
-            distance_strategy, segment_strategy, join_strategy,
-            end_strategy, point_strategy,
-            robust_policy, strategies.side()); // pass strategies?
+                 distance_strategy, segment_strategy, join_strategy,
+                 end_strategy, point_strategy,
+                 robust_policy, strategies);
 
     collection.get_turns();
-    if (BOOST_GEOMETRY_CONDITION(areal))
+    if BOOST_GEOMETRY_CONSTEXPR (areal)
     {
         collection.check_turn_in_original();
     }
@@ -989,7 +989,7 @@ inline void buffer_inserter(GeometryInput const& geometry_input, OutputIterator
     // phase 1: turns (after enrichment/clustering)
     visit_pieces_policy.apply(const_collection, 1);
 
-    if (BOOST_GEOMETRY_CONDITION(areal))
+    if BOOST_GEOMETRY_CONSTEXPR (areal)
     {
         collection.deflate_check_turns();
     }
@@ -1001,8 +1001,7 @@ inline void buffer_inserter(GeometryInput const& geometry_input, OutputIterator
     // - the output is counter clockwise
     // and avoid reversing twice
     bool reverse = distance_strategy.negative() && areal;
-    if (BOOST_GEOMETRY_CONDITION(
-            geometry::point_order<GeometryOutput>::value == counterclockwise))
+    if BOOST_GEOMETRY_CONSTEXPR (geometry::point_order<GeometryOutput>::value == counterclockwise)
     {
         reverse = ! reverse;
     }
@@ -1011,9 +1010,12 @@ inline void buffer_inserter(GeometryInput const& geometry_input, OutputIterator
         collection.reverse();
     }
 
-    if (BOOST_GEOMETRY_CONDITION(distance_strategy.negative() && areal))
+    if BOOST_GEOMETRY_CONSTEXPR (areal)
     {
-        collection.discard_nonintersecting_deflated_rings();
+        if (distance_strategy.negative())
+        {
+            collection.discard_nonintersecting_deflated_rings();
+        }
     }
 
     collection.template assign<GeometryOutput>(out);
