@@ -3,7 +3,7 @@
 // R-tree implementation
 //
 // Copyright (c) 2008 Federico J. Fernandez.
-// Copyright (c) 2011-2019 Adam Wulkiewicz, Lodz, Poland.
+// Copyright (c) 2011-2023 Adam Wulkiewicz, Lodz, Poland.
 // Copyright (c) 2020 Caian Benedicto, Campinas, Brazil.
 //
 // This file was modified by Oracle on 2019-2021.
@@ -23,8 +23,8 @@
 
 // Boost
 #include <boost/container/new_allocator.hpp>
-#include <boost/move/move.hpp>
 #include <boost/tuple/tuple.hpp>
+#include <boost/core/invoke_swap.hpp>
 
 // Boost.Geometry
 #include <boost/geometry/core/static_assert.hpp>
@@ -90,7 +90,15 @@
 #include <boost/geometry/index/detail/rtree/query_iterators.hpp>
 
 #ifdef BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL
-// serialization
+#ifndef BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL_SERIALIZATION
+#define BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL_SERIALIZATION
+#endif
+#ifndef BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL_ITERATORS
+#define BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL_ITERATORS
+#endif
+#endif
+
+#ifdef BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL_SERIALIZATION
 #include <boost/geometry/index/detail/serialization.hpp>
 #endif
 
@@ -129,10 +137,9 @@ Predefined algorithms with run-time parameters are:
  \li \c boost::geometry::index::dynamic_rstar.
 
 \par IndexableGetter
-The object of IndexableGetter type translates from Value to Indexable each time
-r-tree requires it. This means that this operation is done for each Value
-access. Therefore the IndexableGetter should return the Indexable by
-a reference type. The Indexable should not be calculated since it could harm
+An object of IndexableGetter type translates from Value to Indexable each time
+r-tree requires it. This operation is done for each Value access.
+The Indexable should not be calculated each time since it could harm
 the performance. The default IndexableGetter can translate all types adapted
 to Point, Box or Segment concepts (called Indexables). Furthermore, it can
 handle <tt>std::pair<Indexable, T></tt>, <tt>std::tuple<Indexable, ...></tt>
@@ -165,8 +172,6 @@ template
 >
 class rtree
 {
-    BOOST_COPYABLE_AND_MOVABLE(rtree)
-
 public:
     /*! \brief The type of Value stored in the container. */
     typedef Value value_type;
@@ -264,10 +269,10 @@ private:
         members_holder(IndGet const& ind_get,
                        ValEq const& val_eq,
                        Parameters const& parameters,
-                       BOOST_FWD_REF(Alloc) alloc)
+                       Alloc&& alloc)
             : translator_type(ind_get, val_eq)
             , Parameters(parameters)
-            , allocators_type(boost::forward<Alloc>(alloc))
+            , allocators_type(std::forward<Alloc>(alloc))
             , values_count(0)
             , leafs_level(0)
             , root(0)
@@ -301,7 +306,7 @@ private:
         node_pointer root;
     };
 
-    typedef typename members_holder::translator_type translator_type;    
+    typedef typename members_holder::translator_type translator_type;
     typedef typename members_holder::options_type options_type;
     typedef typename members_holder::allocators_type allocators_type;
     typedef typename members_holder::node node;
@@ -312,7 +317,7 @@ private:
     typedef typename members_holder::allocator_traits_type allocator_traits_type;
 
     friend class detail::rtree::utilities::view<rtree>;
-#ifdef BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL
+#ifdef BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL_SERIALIZATION
     friend class detail::rtree::private_view<rtree>;
     friend class detail::rtree::const_private_view<rtree>;
 #endif
@@ -598,7 +603,7 @@ public:
     /*!
     \brief  The copy constructor.
 
-    It uses parameters, translator and allocator from the source tree.
+    It uses parameters, observers and allocator from the source tree.
 
     \param src          The rtree which content will be copied.
 
@@ -619,7 +624,7 @@ public:
     /*!
     \brief The copy constructor.
 
-    It uses Parameters and translator from the source tree.
+    It uses parameters and observers from the source tree.
 
     \param src          The rtree which content will be copied.
     \param allocator    The allocator which will be used.
@@ -640,28 +645,28 @@ public:
     /*!
     \brief The moving constructor.
 
-    It uses parameters, translator and allocator from the source tree.
+    It uses parameters, observers and allocator from the source tree.
 
     \param src          The rtree which content will be moved.
 
     \par Throws
     Nothing.
     */
-    inline rtree(BOOST_RV_REF(rtree) src)
+    inline rtree(rtree&& src)
         : m_members(src.m_members.indexable_getter(),
                     src.m_members.equal_to(),
                     src.m_members.parameters(),
-                    boost::move(src.m_members.allocators()))
+                    std::move(src.m_members.allocators()))
     {
-        boost::swap(m_members.values_count, src.m_members.values_count);
-        boost::swap(m_members.leafs_level, src.m_members.leafs_level);
-        boost::swap(m_members.root, src.m_members.root);
+        boost::core::invoke_swap(m_members.values_count, src.m_members.values_count);
+        boost::core::invoke_swap(m_members.leafs_level, src.m_members.leafs_level);
+        boost::core::invoke_swap(m_members.root, src.m_members.root);
     }
 
     /*!
     \brief The moving constructor.
 
-    It uses parameters and translator from the source tree.
+    It uses parameters and observers from the source tree.
 
     \param src          The rtree which content will be moved.
     \param allocator    The allocator.
@@ -671,7 +676,7 @@ public:
     \li If Value copy constructor throws (only if allocators aren't equal).
     \li If allocation throws or returns invalid value (only if allocators aren't equal).
     */
-    inline rtree(BOOST_RV_REF(rtree) src, allocator_type const& allocator)
+    inline rtree(rtree&& src, allocator_type const& allocator)
         : m_members(src.m_members.indexable_getter(),
                     src.m_members.equal_to(),
                     src.m_members.parameters(),
@@ -679,9 +684,9 @@ public:
     {
         if ( src.m_members.allocators() == allocator )
         {
-            boost::swap(m_members.values_count, src.m_members.values_count);
-            boost::swap(m_members.leafs_level, src.m_members.leafs_level);
-            boost::swap(m_members.root, src.m_members.root);
+            boost::core::invoke_swap(m_members.values_count, src.m_members.values_count);
+            boost::core::invoke_swap(m_members.leafs_level, src.m_members.leafs_level);
+            boost::core::invoke_swap(m_members.root, src.m_members.root);
         }
         else
         {
@@ -692,7 +697,7 @@ public:
     /*!
     \brief The assignment operator.
 
-    It uses parameters and translator from the source tree.
+    It uses parameters and observers from the source tree.
 
     \param src          The rtree which content will be copied.
 
@@ -701,7 +706,7 @@ public:
     \li If allocation throws.
     \li If allocation throws or returns invalid value.
     */
-    inline rtree & operator=(BOOST_COPY_ASSIGN_REF(rtree) src)
+    inline rtree & operator=(rtree const& src)
     {
         if ( &src != this )
         {
@@ -711,11 +716,11 @@ public:
             // NOTE: if propagate is true for std allocators on darwin 4.2.1, glibc++
             // (allocators stored as base classes of members_holder)
             // copying them changes values_count, in this case it doesn't cause errors since data must be copied
-            
+
             typedef std::integral_constant<bool,
                 allocator_traits_type::propagate_on_container_copy_assignment::value
             > propagate;
-            
+
             if ( propagate::value && !(this_allocs == src_allocs) )
                 this->raw_destroy(*this);
             detail::assign_cond(this_allocs, src_allocs, propagate());
@@ -730,7 +735,7 @@ public:
     /*!
     \brief The moving assignment.
 
-    It uses parameters and translator from the source tree.
+    It uses parameters and observers from the source tree.
 
     \param src          The rtree which content will be moved.
 
@@ -739,13 +744,13 @@ public:
     \li If Value copy constructor throws.
     \li If allocation throws or returns invalid value.
     */
-    inline rtree & operator=(BOOST_RV_REF(rtree) src)
+    inline rtree & operator=(rtree&& src)
     {
         if ( &src != this )
         {
             allocators_type & this_allocs = m_members.allocators();
             allocators_type & src_allocs = src.m_members.allocators();
-            
+
             if ( this_allocs == src_allocs )
             {
                 this->raw_destroy(*this);
@@ -754,14 +759,14 @@ public:
                 m_members.equal_to() = src.m_members.equal_to();
                 m_members.parameters() = src.m_members.parameters();
 
-                boost::swap(m_members.values_count, src.m_members.values_count);
-                boost::swap(m_members.leafs_level, src.m_members.leafs_level);
-                boost::swap(m_members.root, src.m_members.root);
+                boost::core::invoke_swap(m_members.values_count, src.m_members.values_count);
+                boost::core::invoke_swap(m_members.leafs_level, src.m_members.leafs_level);
+                boost::core::invoke_swap(m_members.root, src.m_members.root);
 
                 // NOTE: if propagate is true for std allocators on darwin 4.2.1, glibc++
                 // (allocators stored as base classes of members_holder)
                 // moving them changes values_count
-                
+
                 typedef std::integral_constant<bool,
                     allocator_traits_type::propagate_on_container_move_assignment::value
                 > propagate;
@@ -782,7 +787,7 @@ public:
     /*!
     \brief Swaps contents of two rtrees.
 
-    Parameters, translator and allocators are swapped as well.
+    Parameters, observers and allocators are swapped as well.
 
     \param other    The rtree which content will be swapped with this rtree content.
 
@@ -791,22 +796,22 @@ public:
     */
     void swap(rtree & other)
     {
-        boost::swap(m_members.indexable_getter(), other.m_members.indexable_getter());
-        boost::swap(m_members.equal_to(), other.m_members.equal_to());
-        boost::swap(m_members.parameters(), other.m_members.parameters());
-        
+        boost::core::invoke_swap(m_members.indexable_getter(), other.m_members.indexable_getter());
+        boost::core::invoke_swap(m_members.equal_to(), other.m_members.equal_to());
+        boost::core::invoke_swap(m_members.parameters(), other.m_members.parameters());
+
         // NOTE: if propagate is true for std allocators on darwin 4.2.1, glibc++
         // (allocators stored as base classes of members_holder)
         // swapping them changes values_count
-        
+
         typedef std::integral_constant<bool,
             allocator_traits_type::propagate_on_container_swap::value
         > propagate;
         detail::swap_cond(m_members.allocators(), other.m_members.allocators(), propagate());
 
-        boost::swap(m_members.values_count, other.m_members.values_count);
-        boost::swap(m_members.leafs_level, other.m_members.leafs_level);
-        boost::swap(m_members.root, other.m_members.root);
+        boost::core::invoke_swap(m_members.values_count, other.m_members.values_count);
+        boost::core::invoke_swap(m_members.leafs_level, other.m_members.leafs_level);
+        boost::core::invoke_swap(m_members.root, other.m_members.root);
     }
 
     /*!
@@ -995,7 +1000,7 @@ public:
     Values will be returned only if all predicates are met.
 
     <b>Spatial predicates</b>
-    
+
     Spatial predicates may be generated by one of the functions listed below:
     \li \c boost::geometry::index::contains(),
     \li \c boost::geometry::index::covered_by(),
@@ -1015,7 +1020,7 @@ public:
     \li <tt>! boost::geometry::index::within()</tt>
 
     <b>Satisfies predicate</b>
-    
+
     This is a special kind of predicate which allows to pass a user-defined function or function object which checks
     if Value should be returned by the query. It's generated by:
     \li \c boost::geometry::index::satisfies().
@@ -1026,7 +1031,7 @@ public:
     in returning k values to the output iterator. Only one nearest predicate may be passed to the query.
     It may be generated by:
     \li \c boost::geometry::index::nearest().
-        
+
     <b>Connecting predicates</b>
 
     Predicates may be passed together connected with \c operator&&().
@@ -1091,7 +1096,7 @@ public:
     For the information about predicates which may be passed to this method see query().
 
     \par Example
-    \verbatim    
+    \verbatim
     for ( Rtree::const_query_iterator it = tree.qbegin(bgi::nearest(pt, 10000)) ;
           it != tree.qend() ; ++it )
     {
@@ -1123,7 +1128,7 @@ public:
     The modification of the rtree may invalidate the iterators.
 
     \param predicates   Predicates.
-    
+
     \return             The iterator pointing at the begin of the query range.
     */
     template <typename Predicates>
@@ -1138,7 +1143,7 @@ public:
     This method returns an iterator which may be used to check if the query has ended.
 
     \par Example
-    \verbatim    
+    \verbatim
     for ( Rtree::const_query_iterator it = tree.qbegin(bgi::nearest(pt, 10000)) ;
           it != tree.qend() ; ++it )
     {
@@ -1167,7 +1172,7 @@ public:
 
     \warning
     The modification of the rtree may invalidate the iterators.
-    
+
     \return             The iterator pointing at the end of the query range.
     */
     const_query_iterator qend() const
@@ -1184,7 +1189,9 @@ private:
             detail::rtree::iterators::distance_query_iterator<members_holder, Predicates>
         >;
 
-#ifndef BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL
+#ifdef BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL_ITERATORS
+public:
+#else
 private:
 #endif
     /*!
@@ -1192,7 +1199,7 @@ private:
 
     This method returns an iterator which may be used to perform iterative queries.
     For the information about predicates which may be passed to this method see query().
-    
+
     The type of the returned iterator depends on the type of passed Predicates but the iterator of this type
     may be assigned to the variable of const_query_iterator type. If you'd like to use the type of the iterator
     returned by this method you may get the type e.g. by using C++11 decltype or Boost.Typeof library.
@@ -1237,7 +1244,7 @@ private:
     The modification of the rtree may invalidate the iterators.
 
     \param predicates   Predicates.
-    
+
     \return             The iterator pointing at the begin of the query range.
     */
     template <typename Predicates>
@@ -1257,7 +1264,7 @@ private:
 
     This method returns the iterator which may be used to perform iterative queries. For the information
     about the predicates which may be passed to this method see query().
-    
+
     The type of the returned iterator depends on the type of passed Predicates but the iterator of this type
     may be assigned to the variable of const_query_iterator type. If you'd like to use the type of the iterator
     returned by this method you may get the type e.g. by using C++11 decltype or Boost.Typeof library.
@@ -1281,7 +1288,7 @@ private:
     The modification of the rtree may invalidate the iterators.
 
     \param predicates   Predicates.
-    
+
     \return             The iterator pointing at the end of the query range.
     */
     template <typename Predicates>
@@ -1299,7 +1306,7 @@ private:
 
     This method returns the iterator which may be compared with the iterator returned by qbegin() in order to
     check if the query has ended.
-    
+
     The type of the returned iterator is different than the type returned by qbegin() but the iterator of this type
     may be assigned to the variable of const_query_iterator type. If you'd like to use the type of the iterator returned by this
     method, which most certainly will be faster than the type-erased iterator, you may get the type
@@ -1340,7 +1347,7 @@ private:
 
     \warning
     The modification of the rtree may invalidate the iterators.
-    
+
     \return             The iterator pointing at the end of the query range.
     */
     detail::rtree::iterators::end_query_iterator<value_type, allocators_type>
@@ -1501,7 +1508,7 @@ public:
 
     /*!
     \brief Count Values or Indexables stored in the container.
-    
+
     For indexable_type it returns the number of values which indexables equals the parameter.
     For value_type it returns the number of values which equals the parameter.
 
@@ -1883,7 +1890,7 @@ private:
 
         return distance_v.apply(m_members, out_it);
     }
-    
+
     /*!
     \brief Count elements corresponding to value or indexable.
 
@@ -2072,7 +2079,7 @@ This query function performs spatial and k-nearest neighbor searches. It allows
 Values will be returned only if all predicates are met.
 
 <b>Spatial predicates</b>
-    
+
 Spatial predicates may be generated by one of the functions listed below:
 \li \c boost::geometry::index::contains(),
 \li \c boost::geometry::index::covered_by(),
@@ -2103,7 +2110,7 @@ If the nearest predicate is passed a k-nearest neighbor search will be performed
 in returning k values to the output iterator. Only one nearest predicate may be passed to the query.
 It may be generated by:
 \li \c boost::geometry::index::nearest().
-        
+
 <b>Connecting predicates</b>
 
 Predicates may be passed together connected with \c operator&&().
@@ -2153,7 +2160,7 @@ query(rtree<Value, Parameters, IndexableGetter, EqualTo, Allocator> const& tree,
 
 This method returns the iterator which may be used to perform iterative queries. For the information
 about the predicates which may be passed to this method see query().
-    
+
 \par Example
 \verbatim
 std::for_each(bgi::qbegin(tree, bgi::nearest(pt, 3)), bgi::qend(tree), do_something());
@@ -2173,7 +2180,7 @@ The modification of the rtree may invalidate the iterators.
 
 \param tree         The rtree.
 \param predicates   Predicates.
-    
+
 \return             The iterator pointing at the begin of the query range.
 */
 template <typename Value, typename Parameters, typename IndexableGetter, typename EqualTo, typename Allocator,
@@ -2189,7 +2196,7 @@ qbegin(rtree<Value, Parameters, IndexableGetter, EqualTo, Allocator> const& tree
 \brief Returns the query iterator pointing at the end of the query range.
 
 This method returns the iterator which may be used to check if the query has ended.
-    
+
 \par Example
 \verbatim
 std::for_each(bgi::qbegin(tree, bgi::nearest(pt, 3)), bgi::qend(tree), do_something());
@@ -2205,7 +2212,7 @@ Nothing
 The modification of the rtree may invalidate the iterators.
 
 \ingroup rtree_functions
-    
+
 \return             The iterator pointing at the end of the query range.
 */
 template <typename Value, typename Parameters, typename IndexableGetter, typename EqualTo, typename Allocator> inline
