@@ -23,7 +23,8 @@
 #include <type_traits>
 #include <utility>
 
-BOOST_JSON_NS_BEGIN
+namespace boost {
+namespace json {
 
 class value;
 class value_ref;
@@ -85,7 +86,7 @@ class object
     template<class T>
     using is_inputit = typename std::enable_if<
         std::is_constructible<key_value_pair,
-        typename std::iterator_traits<T>::value_type
+        typename std::iterator_traits<T>::reference
             >::value>::type;
 
     BOOST_JSON_DECL
@@ -171,13 +172,15 @@ public:
     /** Default constructor.
 
         The constructed object is empty with zero
-        capacity, using the default memory resource.
+        capacity, using the [default memory resource].
 
         @par Complexity
         Constant.
 
         @par Exception Safety
         No-throw guarantee.
+
+        [default memory resource]: json/allocators/storage_ptr.html#json.allocators.storage_ptr.default_memory_resource
     */
     object() noexcept
         : t_(&empty_)
@@ -246,7 +249,7 @@ public:
         @code
         std::is_constructible_v<
             key_value_pair,
-            std::iterator_traits<InputIt>::value_type>
+            std::iterator_traits<InputIt>::reference>
         @endcode
 
         @par Complexity
@@ -893,15 +896,11 @@ public:
 
         @param new_capacity The new minimum capacity.
 
-        @throw std::length_error `new_capacity > max_size()`
+        @throw system_error `new_capacity > max_size()`
     */
+    inline
     void
-    reserve(std::size_t new_capacity)
-    {
-        if(new_capacity <= capacity())
-            return;
-        rehash(new_capacity);
-    }
+    reserve(std::size_t new_capacity);
 
     //------------------------------------------------------
     //
@@ -929,8 +928,14 @@ public:
 
     /** Insert elements.
 
-        Inserts `p`, from which @ref value_type must
-        be constructible.
+        Inserts `p` if `this->contains(value_type(p).key())` is `false`.
+        @ref value_type must be constructible from `p`.
+
+        If the insertion occurs and results in a rehashing
+        of the container, all iterators and references are invalidated.
+        Otherwise, they are not affected.
+        Rehashing occurs only if the new number of elements
+        is greater than @ref capacity().
 
         @par Constraints
         @code
@@ -947,14 +952,13 @@ public:
 
         @param p The value to insert.
 
-        @throw std::length_error key is too long.
+        @throw system_error key is too long.
 
-        @throw std::length_error @ref size() >= max_size().
+        @throw system_error @ref size() >= max_size().
 
         @return A pair where `first` is an iterator
         to the existing or inserted element, and `second`
-        is `true` if the insertion took place or `false` if
-        the assignment took place.
+        is `true` if the insertion took place or `false` otherwise.
     */
     template<class P
 #ifndef BOOST_JSON_DOCS
@@ -968,26 +972,31 @@ public:
 
     /** Insert elements.
 
-        The elements in the range `{first, last)` whose
-        keys are unique are inserted one at a time, in order.
-        If there are elements with duplicate keys; that
-        is, if multiple elements in the range have keys
-        that compare equal, only the first equivalent
-        element will be inserted.
+        The elements in the range `[first, last)` are inserted one at a time,
+        in order. Any element with key that is a duplicate of a key already
+        present in container will be skipped. This also means, that if there
+        are two keys within the range that are equal to each other, only the
+        first will be inserted.
+
+        Insertion may result in rehashing of the container. In that case all
+        iterators and references are invalidated. Otherwise, they are not
+        affected.
 
         @par Precondition
         `first` and `last` are not iterators into `*this`.
+        `first` and `last` form a valid range.
 
         @par Constraints
         @code
-        std::is_constructible_v<value_type, std::iterator_traits<InputIt>::value_type>
+        std::is_constructible_v<value_type, std::iterator_traits<InputIt>::reference>
         @endcode
 
         @par Complexity
         Linear in `std::distance(first, last)`.
 
         @par Exception Safety
-        Strong guarantee.
+        Strong guarantee for forward iterators, basic guarantee for input
+        iterators.
         Calls to `memory_resource::allocate` may throw.
 
         @param first An input iterator pointing to the first
@@ -1015,18 +1024,21 @@ public:
 
     /** Insert elements.
 
-        The elements in the initializer list whose
-        keys are unique are inserted one at a time, in order.
-        If there are elements with duplicate keys; that
-        is, if multiple elements in the range have keys
-        that compare equal, only the first equivalent
-        element will be inserted.
+        The elements in the initializer list are inserted one at a time, in
+        order. Any element with key that is a duplicate of a key already
+        present in container will be skipped. This also means, that if there
+        are two keys within the initializer list that are equal to each other,
+        only the first will be inserted.
+
+        Insertion may result in rehashing of the container. In that case all
+        iterators and references are invalidated. Otherwise, they are not
+        affected.
 
         @par Complexity
         Linear in `init.size()`.
 
         @par Exception Safety
-        Strong guarantee.
+        Basic guarantee.
         Calls to `memory_resource::allocate` may throw.
 
         @param init The initializer list to insert
@@ -1039,18 +1051,15 @@ public:
     /** Insert an element or assign to the current element if the key already exists.
 
         If the key equivalent to `key` already exists in the
-        container. assigns `std::forward<M>(obj)` to the
-        `mapped type` corresponding to the key. Otherwise,
+        container, assigns `std::forward<M>(m)` to the
+        `mapped_type` corresponding to the key. Otherwise,
         inserts the new value at the end as if by insert,
-        constructing it from
-        `value_type(key, std::forward<M>(obj))`.
+        constructing it from `value_type(key, std::forward<M>(m))`.
 
-        If the insertion occurs and results in a rehashing
-        of the container, all iterators are invalidated.
-        Otherwise, iterators are not affected.
-        References are not invalidated.
-        Rehashing occurs only if the new number of elements
-        is greater than @ref capacity().
+        If the insertion occurs and results in a rehashing of the container,
+        all iterators and references are invalidated. Otherwise, they are not
+        affected. Rehashing occurs only if the new number of elements is
+        greater than @ref capacity().
 
         @par Complexity
         Amortized constant on average, worst case linear in @ref size().
@@ -1068,7 +1077,7 @@ public:
 
         @param m The value to insert or assign
 
-        @throw std::length_error if key is too long
+        @throw system_error if key is too long
     */
     template<class M>
     std::pair<iterator, bool>
@@ -1079,16 +1088,12 @@ public:
 
         Inserts a new element into the container constructed
         in-place with the given argument if there is no
-        element with the key in the container.
-        The element is inserted after all the existing
-        elements.
+        element with the `key` in the container.
 
-        If the insertion occurs and results in a rehashing
-        of the container, all iterators are invalidated.
-        Otherwise, iterators are not affected.
-        References are not invalidated.
-        Rehashing occurs only if the new number of elements
-        is greater than @ref capacity().
+        If the insertion occurs and results in a rehashing of the container,
+        all iterators and references are invalidated. Otherwise, they are not
+        affected. Rehashing occurs only if the new number of elements is
+        greater than @ref capacity().
 
         @par Complexity
         Amortized constant on average, worst case linear in @ref size().
@@ -1099,8 +1104,7 @@ public:
 
         @return A `std::pair` where `first` is an iterator
         to the existing or inserted element, and `second`
-        is `true` if the insertion took place or `false` if
-        the assignment took place.
+        is `true` if the insertion took place or `false` otherwise.
 
         @param key The key used for lookup and insertion
 
@@ -1108,7 +1112,7 @@ public:
         This will be passed as `std::forward<Arg>(arg)` to
         the @ref value constructor.
 
-        @throw std::length_error if key is too long
+        @throw system_error if key is too long
     */
     template<class Arg>
     std::pair<iterator, bool>
@@ -1117,20 +1121,23 @@ public:
     /** Erase an element
 
         Remove the element pointed to by `pos`, which must
-        be valid and dereferenceable. Thus the @ref end()
-        iterator (which is valid but cannot be dereferenced)
-        cannot be used as a value for `pos`.
+        be valid and dereferenceable.
         References and iterators to the erased element are
         invalidated. Other iterators and references are not
         invalidated.
 
+        @note
+
+        The @ref end() iterator (which is valid but cannot be
+        dereferenced) cannot be used as a value for `pos`.
+
         @par Complexity
         Constant on average, worst case linear in @ref size().
 
         @par Exception Safety
         No-throw guarantee.
 
-        @return An iterator following the last removed element.
+        @return An iterator following the removed element.
 
         @param pos An iterator pointing to the element to be
         removed.
@@ -1161,6 +1168,56 @@ public:
     std::size_t
     erase(string_view key) noexcept;
 
+    /** Erase an element preserving order
+
+        Remove the element pointed to by `pos`, which must
+        be valid and dereferenceable.
+        References and iterators from `pos` to `end()`, both
+        included, are invalidated. Other iterators and references
+        are not invalidated.
+        The relative order of remaining elements is preserved.
+
+        @note
+
+        The @ref end() iterator (which is valid but cannot be
+        dereferenced) cannot be used as a value for `pos`.
+
+        @par Complexity
+        Linear in @ref size().
+
+        @par Exception Safety
+        No-throw guarantee.
+
+        @return An iterator following the removed element.
+
+        @param pos An iterator pointing to the element to be
+        removed.
+    */
+    BOOST_JSON_DECL
+    iterator
+    stable_erase(const_iterator pos) noexcept;
+
+    /** Erase an element preserving order
+
+        Remove the element which matches `key`, if it exists.
+        All references and iterators are invalidated.
+        The relative order of remaining elements is preserved.
+
+        @par Complexity
+        Linear in @ref size().
+
+        @par Exception Safety
+        No-throw guarantee.
+
+        @return The number of elements removed, which will
+        be either 0 or 1.
+
+        @param key The key to match.
+    */
+    BOOST_JSON_DECL
+    std::size_t
+    stable_erase(string_view key) noexcept;
+
     /** Swap two objects.
 
         Exchanges the contents of this object with another
@@ -1254,32 +1311,21 @@ public:
 
         @param key The key of the element to find.
 
-        @throw std::out_of_range if no such element exists.
+        @throw system_error if no such element exists.
     */
+    /* @{ */
     inline
     value&
-    at(string_view key);
+    at(string_view key) &;
 
-    /** Access the specified element, with bounds checking.
+    inline
+    value&&
+    at(string_view key) &&;
 
-        Returns a constant reference to the mapped value of
-        the element that matches `key`, otherwise throws.
-
-        @par Complexity
-        Constant on average, worst case linear in @ref size().
-
-        @par Exception Safety
-        Strong guarantee.
-
-        @return A reference to the mapped value.
-
-        @param key The key of the element to find.
-
-        @throw std::out_of_range if no such element exists.
-    */
     inline
     value const&
-    at(string_view key) const;
+    at(string_view key) const&;
+    /* @} */
 
     /** Access or insert the specified element
 
@@ -1482,6 +1528,29 @@ public:
         return ! (lhs == rhs);
     }
 
+    /** Serialize @ref object to an output stream.
+
+        This function serializes an `object` as JSON into the output stream.
+
+        @return Reference to `os`.
+
+        @par Complexity
+        Constant or linear in the size of `obj`.
+
+        @par Exception Safety
+        Strong guarantee.
+        Calls to `memory_resource::allocate` may throw.
+
+        @param os The output stream to serialize to.
+
+        @param obj The value to serialize.
+    */
+    BOOST_JSON_DECL
+    friend
+    std::ostream&
+    operator<<(
+        std::ostream& os,
+        object const& obj);
 private:
 #ifndef BOOST_JSON_DOCS
     // VFALCO friending a detail function makes it public
@@ -1523,10 +1592,9 @@ private:
         InputIt last,
         std::forward_iterator_tag);
 
-    BOOST_JSON_DECL
+    template< class... Args >
     std::pair<iterator, bool>
-    insert_impl(
-        pilfered<key_value_pair> p);
+    emplace_impl(string_view key, Args&& ... args );
 
     BOOST_JSON_DECL
     key_value_pair*
@@ -1535,8 +1603,8 @@ private:
         std::size_t hash);
 
     BOOST_JSON_DECL
-    void
-    rehash(std::size_t new_capacity);
+    table*
+    reserve_impl(std::size_t new_capacity);
 
     BOOST_JSON_DECL
     bool
@@ -1562,12 +1630,43 @@ private:
     destroy(
         key_value_pair* first,
         key_value_pair* last) noexcept;
+
+    template<class FS, class FB>
+    auto
+    do_erase(
+        const_iterator pos,
+        FS small_reloc,
+        FB big_reloc) noexcept
+        -> iterator;
+
+    inline
+    void
+    reindex_relocate(
+        key_value_pair* src,
+        key_value_pair* dst) noexcept;
 };
 
-BOOST_JSON_NS_END
+} // namespace json
+} // namespace boost
+
+#ifndef BOOST_JSON_DOCS
+// boost::hash trait
+namespace boost
+{
+namespace container_hash
+{
+
+template< class T > struct is_unordered_range;
+
+template<>
+struct is_unordered_range< json::object >
+    : std::true_type
+{};
+
+} // namespace container_hash
+} // namespace boost
 
 // std::hash specialization
-#ifndef BOOST_JSON_DOCS
 namespace std {
 template <>
 struct hash< ::boost::json::object > {
