@@ -1,24 +1,18 @@
 #ifndef BOOST_LEAF_ERROR_HPP_INCLUDED
 #define BOOST_LEAF_ERROR_HPP_INCLUDED
 
-// Copyright 2018-2022 Emil Dotchevski and Reverge Studios, Inc.
+// Copyright 2018-2023 Emil Dotchevski and Reverge Studios, Inc.
 
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
 #include <boost/leaf/config.hpp>
 #include <boost/leaf/detail/optional.hpp>
-#include <boost/leaf/detail/demangle.hpp>
 #include <boost/leaf/detail/function_traits.hpp>
 #include <boost/leaf/detail/print.hpp>
 
-#include <type_traits>
-#include <iosfwd>
-
 #if BOOST_LEAF_CFG_DIAGNOSTICS
-#   include <sstream>
-#   include <string>
-#   include <set>
+#   include <ostream>
 #endif
 
 #if BOOST_LEAF_CFG_STD_SYSTEM_ERROR
@@ -47,25 +41,25 @@
 #if BOOST_LEAF_CFG_GNUC_STMTEXPR
 
 #define BOOST_LEAF_CHECK(r)\
-    ({\
-        auto && BOOST_LEAF_TMP = (r);\
-        static_assert(::boost::leaf::is_result_type<typename std::decay<decltype(BOOST_LEAF_TMP)>::type>::value,\
-            "BOOST_LEAF_CHECK requires a result object (see is_result_type)");\
-        if( !BOOST_LEAF_TMP )\
-            return BOOST_LEAF_TMP.error();\
-        std::move(BOOST_LEAF_TMP);\
-    }).value()
+        ({\
+            auto && BOOST_LEAF_TMP = (r);\
+            static_assert(::boost::leaf::is_result_type<typename std::decay<decltype(BOOST_LEAF_TMP)>::type>::value,\
+                "BOOST_LEAF_CHECK requires a result object (see is_result_type)");\
+            if( !BOOST_LEAF_TMP )\
+                return BOOST_LEAF_TMP.error();\
+            std::move(BOOST_LEAF_TMP);\
+        }).value()
 
 #else
 
 #define BOOST_LEAF_CHECK(r)\
-    {\
-        auto && BOOST_LEAF_TMP = (r);\
-        static_assert(::boost::leaf::is_result_type<typename std::decay<decltype(BOOST_LEAF_TMP)>::type>::value,\
-            "BOOST_LEAF_CHECK requires a result object (see is_result_type)");\
-        if( !BOOST_LEAF_TMP )\
-            return BOOST_LEAF_TMP.error();\
-    }
+        {\
+            auto && BOOST_LEAF_TMP = (r);\
+            static_assert(::boost::leaf::is_result_type<typename std::decay<decltype(BOOST_LEAF_TMP)>::type>::value,\
+                "BOOST_LEAF_CHECK requires a result object (see is_result_type)");\
+            if( !BOOST_LEAF_TMP )\
+                return BOOST_LEAF_TMP.error();\
+        }
 
 #endif
 
@@ -112,8 +106,8 @@ namespace leaf_detail
         char const * (*first_type)();
         int count;
 
-        BOOST_LEAF_CONSTEXPR explicit e_unexpected_count(char const * (*first_type)()) noexcept:
-            first_type(first_type),
+        BOOST_LEAF_CONSTEXPR explicit e_unexpected_count(char const * (*ft)()) noexcept:
+            first_type(ft),
             count(1)
         {
         }
@@ -141,33 +135,117 @@ namespace leaf_detail
         BOOST_LEAF_CONSTEXPR static void print( std::basic_ostream<CharT, Traits> &, e_unexpected_count const &) noexcept { }
     };
 
+    template <class E>
+    class BOOST_LEAF_SYMBOL_VISIBLE slot;
+
     class BOOST_LEAF_SYMBOL_VISIBLE e_unexpected_info
     {
-        std::string s_;
-        std::set<char const *(*)()> already_;
+        e_unexpected_info( e_unexpected_info const & ) = delete;
+        e_unexpected_info & operator=( e_unexpected_info const & ) = delete;
+
+        struct slot_base
+        {
+        public:
+            slot_base * next_;
+            virtual void deactivate() noexcept = 0;
+            virtual void propagate( int err_id ) noexcept = 0;
+            virtual void print( std::ostream &, int key_to_print ) const = 0;
+            virtual ~slot_base() noexcept { };
+        protected:
+            BOOST_LEAF_CONSTEXPR slot_base():
+                next_(nullptr)
+            {
+            }
+        };
+
+        template <class E>
+        class slot_store: public slot_base, public slot<E>
+        {
+            slot_store( slot_store const & ) = delete;
+            slot_store & operator=( slot_store const & ) = delete;
+            void deactivate() noexcept final override
+            {
+                slot<E>::deactivate();
+            }
+            void propagate( int err_id ) noexcept final override
+            {
+                slot<E>::propagate(err_id);
+            }
+            void print( std::ostream & os, int key_to_print ) const final override
+            {
+                slot<E>::print(os, key_to_print);
+            }
+        public:
+            template <class T>
+            BOOST_LEAF_CONSTEXPR slot_store( int err_id, T && e )
+            {
+                slot<E>::put(err_id, std::forward<T>(e));
+            }
+        };
+
+        slot_base * first_;
+        slot_base * * last_;
 
     public:
 
-        e_unexpected_info() noexcept
+        BOOST_LEAF_CONSTEXPR e_unexpected_info() noexcept:
+            first_(nullptr),
+            last_(&first_)
         {
         }
 
-        template <class E>
-        void add(E && e)
+        BOOST_LEAF_CONSTEXPR e_unexpected_info( e_unexpected_info && other ) noexcept:
+            first_(other.first_),
+            last_(other.last_ == &other.first_? &first_ : other.last_)
         {
-            if( !diagnostic<E>::is_invisible && already_.insert(&type<E>).second  )
+            BOOST_LEAF_ASSERT(last_ != nullptr);
+            BOOST_LEAF_ASSERT(*last_ == nullptr);
+            other.first_ = nullptr;
+            other.last_ = nullptr;
+        }
+
+        ~e_unexpected_info() noexcept
+        {
+            for( slot_base const * p = first_; p; )
             {
-                std::stringstream s;
-                diagnostic<E>::print(s,e);
-                (s << '\n').flush();
-                s_ += s.str();
+                slot_base const * n = p -> next_;
+                delete p;
+                p = n;
             }
         }
 
-        template <class CharT, class Traits>
-        void print( std::basic_ostream<CharT, Traits> & os ) const
+        template <class E>
+        BOOST_LEAF_CONSTEXPR typename std::decay<E>::type & put(int err_id, E && e)
         {
-            os << "Unhandled error objects:\n" << s_;
+            using T = typename std::decay<E>::type;
+            BOOST_LEAF_ASSERT(last_ != nullptr);
+            BOOST_LEAF_ASSERT(*last_ == nullptr);
+            BOOST_LEAF_ASSERT(tls::read_ptr<slot<T>>() == nullptr);
+            slot_store<T> * ss = new slot_store<T>(err_id, std::forward<E>(e));
+            *last_ = ss;
+            last_ = &ss->next_;
+            ss->activate();
+            return ss->value(err_id);
+        }
+
+        void deactivate() noexcept
+        {
+            for( slot_base * p=first_; p; p=p->next_ )
+                p->deactivate();
+        }
+
+        void propagate( int err_id ) noexcept
+        {
+            for( slot_base * p=first_; p; p=p->next_ )
+                p->propagate(err_id);
+        }
+
+        template <class CharT, class Traits>
+        void print( std::basic_ostream<CharT, Traits> & os, int key_to_print ) const
+        {
+            os << "Unhandled error objects:\n";
+            for( slot_base const * p=first_; p; p=p->next_ )
+                p->print(os, key_to_print);
         }
     };
 
@@ -197,7 +275,7 @@ struct BOOST_LEAF_SYMBOL_VISIBLE e_source_location
     char const * function;
 
     template <class CharT, class Traits>
-    friend std::basic_ostream<CharT, Traits> & operator<<( std::basic_ostream<CharT, Traits> & os, e_source_location const & x )
+    friend std::ostream & operator<<( std::basic_ostream<CharT, Traits> & os, e_source_location const & x )
     {
         return os << leaf::type<e_source_location>() << ": " << x.file << '(' << x.line << ") in function " << x.function;
     }
@@ -231,18 +309,18 @@ namespace leaf_detail
             BOOST_LEAF_ASSERT(x.prev_==nullptr);
         }
 
-        BOOST_LEAF_CONSTEXPR void activate() noexcept
+        void activate() noexcept
         {
             prev_ = tls::read_ptr<slot<E>>();
             tls::write_ptr<slot<E>>(this);
         }
 
-        BOOST_LEAF_CONSTEXPR void deactivate() noexcept
+        void deactivate() noexcept
         {
             tls::write_ptr<slot<E>>(prev_);
         }
 
-        BOOST_LEAF_CONSTEXPR void propagate( int err_id ) noexcept;
+        void propagate( int err_id ) noexcept(!BOOST_LEAF_CFG_DIAGNOSTICS);
 
         template <class CharT, class Traits>
         void print( std::basic_ostream<CharT, Traits> & os, int key_to_print ) const
@@ -257,10 +335,13 @@ namespace leaf_detail
                             return;
                     }
                     else
-                        os << '[' << k << ']';
+                        os << '[' << k << "] ";
                     diagnostic<E>::print(os, value(k));
                     (os << '\n').flush();
                 }
+#else
+            (void) os;
+            (void) key_to_print;
 #endif
         }
 
@@ -271,6 +352,22 @@ namespace leaf_detail
 
 #if BOOST_LEAF_CFG_DIAGNOSTICS
 
+    template <>
+    inline void slot<e_unexpected_info>::deactivate() noexcept
+    {
+        if( int const err_id = this->key() )
+            if( e_unexpected_info * info = this->has_value(err_id) )
+                info->deactivate();
+        tls::write_ptr<slot<e_unexpected_info>>(prev_);
+    }
+
+    template <>
+    inline void slot<e_unexpected_info>::propagate( int err_id )
+    {
+        if( e_unexpected_info * info = this->has_value(err_id) )
+            info->propagate(err_id);
+    }
+
     template <class E>
     BOOST_LEAF_CONSTEXPR inline void load_unexpected_count( int err_id ) noexcept
     {
@@ -284,28 +381,77 @@ namespace leaf_detail
     }
 
     template <class E>
-    BOOST_LEAF_CONSTEXPR inline void load_unexpected_info( int err_id, E && e ) noexcept
+    BOOST_LEAF_CONSTEXPR inline void load_unexpected_info( int err_id, E && e )
     {
         if( slot<e_unexpected_info> * sl = tls::read_ptr<slot<e_unexpected_info>>() )
         {
             if( e_unexpected_info * unx = sl->has_value(err_id) )
-                unx->add(std::forward<E>(e));
+                (void) unx->put(err_id, std::forward<E>(e));
             else
-                sl->put(err_id, e_unexpected_info()).add(std::forward<E>(e));
+                (void) sl->put(err_id).put(err_id, std::forward<E>(e));
         }
     }
 
-    template <class E>
-    BOOST_LEAF_CONSTEXPR inline void load_unexpected( int err_id, E && e  ) noexcept
+    template <class E, class F>
+    BOOST_LEAF_CONSTEXPR inline void accumulate_unexpected_info( int err_id, F && f )
+    {
+        if( slot<e_unexpected_info> * sl = tls::read_ptr<slot<e_unexpected_info>>() )
+        {
+            if( e_unexpected_info * unx = sl->has_value(err_id) )
+                (void) std::forward<F>(f)(unx->put(err_id, E{}));
+            else
+                (void) std::forward<F>(f)(sl->put(err_id).put(err_id, E{}));
+        }
+    }
+
+    template <bool OnError, class E>
+    inline void load_unexpected( int err_id, E && e  ) noexcept(OnError)
     {
         load_unexpected_count<E>(err_id);
-        load_unexpected_info(err_id, std::forward<E>(e));
+        if( OnError )
+        {
+#ifndef BOOST_LEAF_NO_EXCEPTIONS
+            try
+            {
+#endif
+                load_unexpected_info(err_id, std::forward<E>(e));
+#ifndef BOOST_LEAF_NO_EXCEPTIONS
+            }
+            catch(...)
+            {
+            }
+#endif
+        }
+        else
+            load_unexpected_info(err_id, std::forward<E>(e));
+    }
+
+    template <bool OnError, class E, class F>
+    inline void accumulate_unexpected( int err_id, F && f  ) noexcept(OnError)
+    {
+        load_unexpected_count<E>(err_id);
+        if( OnError )
+        {
+#ifndef BOOST_LEAF_NO_EXCEPTIONS
+            try
+            {
+#endif
+                accumulate_unexpected_info<E>(err_id, std::forward<F>(f));
+#ifndef BOOST_LEAF_NO_EXCEPTIONS
+            }
+            catch(...)
+            {
+            }
+#endif
+        }
+        else
+            accumulate_unexpected_info<E>(err_id, std::forward<F>(f));
     }
 
 #endif
 
     template <class E>
-    BOOST_LEAF_CONSTEXPR inline void slot<E>::propagate( int err_id ) noexcept
+    inline void slot<E>::propagate( int err_id ) noexcept(!BOOST_LEAF_CFG_DIAGNOSTICS)
     {
         if( this->key()!=err_id && err_id!=0 )
             return;
@@ -314,37 +460,40 @@ namespace leaf_detail
 #if BOOST_LEAF_CFG_DIAGNOSTICS
         else
         {
-            int c = tls::read_uint32<tls_tag_unexpected_enabled_counter>();
+            int c = int(tls::read_uint<tls_tag_unexpected_enabled_counter>());
             BOOST_LEAF_ASSERT(c>=0);
             if( c )
-                load_unexpected(err_id, std::move(*this).value(err_id));
+                load_unexpected<false>(err_id, std::move(*this).value(err_id));
         }
 #endif
     }
 
-    template <class E>
-    BOOST_LEAF_CONSTEXPR inline int load_slot( int err_id, E && e ) noexcept
+    template <bool OnError, class E>
+    BOOST_LEAF_CONSTEXPR inline int load_slot( int err_id, E && e ) noexcept(OnError)
     {
         static_assert(!std::is_pointer<E>::value, "Error objects of pointer types are not allowed");
         static_assert(!std::is_same<typename std::decay<E>::type, error_id>::value, "Error objects of type error_id are not allowed");
         using T = typename std::decay<E>::type;
         BOOST_LEAF_ASSERT((err_id&3)==1);
         if( slot<T> * p = tls::read_ptr<slot<T>>() )
-            (void) p->put(err_id, std::forward<E>(e));
+        {
+            if( !OnError || !p->has_value(err_id) )
+                (void) p->put(err_id, std::forward<E>(e));
+        }
 #if BOOST_LEAF_CFG_DIAGNOSTICS
         else
         {
-            int c = tls::read_uint32<tls_tag_unexpected_enabled_counter>();
+            int c = int(tls::read_uint<tls_tag_unexpected_enabled_counter>());
             BOOST_LEAF_ASSERT(c>=0);
             if( c )
-                load_unexpected(err_id, std::forward<E>(e));
+                load_unexpected<OnError>(err_id, std::forward<E>(e));
         }
 #endif
         return 0;
     }
 
-    template <class F>
-    BOOST_LEAF_CONSTEXPR inline int accumulate_slot( int err_id, F && f ) noexcept
+    template <bool OnError, class F>
+    BOOST_LEAF_CONSTEXPR inline int accumulate_slot( int err_id, F && f ) noexcept(OnError)
     {
         static_assert(function_traits<F>::arity==1, "Lambdas passed to accumulate must take a single e-type argument by reference");
         using E = typename std::decay<fn_arg_type<F,0>>::type;
@@ -357,6 +506,15 @@ namespace leaf_detail
             else
                 (void) std::forward<F>(f)(sl->put(err_id,E()));
         }
+#if BOOST_LEAF_CFG_DIAGNOSTICS
+        else
+        {
+            int c = int(tls::read_uint<tls_tag_unexpected_enabled_counter>());
+            BOOST_LEAF_ASSERT(c>=0);
+            if( c )
+                accumulate_unexpected<OnError, E>(err_id, std::forward<F>(f));
+        }
+#endif
         return 0;
     }
 }
@@ -379,20 +537,20 @@ namespace leaf_detail
     };
 
     template <class T>
-    atomic_unsigned_int id_factory<T>::counter(-3);
+    atomic_unsigned_int id_factory<T>::counter(unsigned(-3));
 
     inline int current_id() noexcept
     {
-        auto id = tls::read_uint32<tls_tag_id_factory_current_id>();
+        unsigned id = tls::read_uint<tls_tag_id_factory_current_id>();
         BOOST_LEAF_ASSERT(id==0 || (id&3)==1);
-        return id;
+        return int(id);
     }
 
     inline int new_id() noexcept
     {
-        auto id = id_factory<>::generate_next_id();
-        tls::write_uint32<tls_tag_id_factory_current_id>(id);
-        return id;
+        unsigned id = id_factory<>::generate_next_id();
+        tls::write_uint<tls_tag_id_factory_current_id>(id);
+        return int(id);
     }
 }
 
@@ -409,27 +567,27 @@ namespace leaf_detail
     template <class E>
     struct load_item<E, -1>
     {
-        BOOST_LEAF_CONSTEXPR static int load( int err_id, E && e ) noexcept
+        BOOST_LEAF_CONSTEXPR static int load_( int err_id, E && e ) noexcept
         {
-            return load_slot(err_id, std::forward<E>(e));
+            return load_slot<false>(err_id, std::forward<E>(e));
         }
     };
 
     template <class F>
     struct load_item<F, 0>
     {
-        BOOST_LEAF_CONSTEXPR static int load( int err_id, F && f ) noexcept
+        BOOST_LEAF_CONSTEXPR static int load_( int err_id, F && f ) noexcept
         {
-            return load_slot(err_id, std::forward<F>(f)());
+            return load_slot<false>(err_id, std::forward<F>(f)());
         }
     };
 
     template <class F>
     struct load_item<F, 1>
     {
-        BOOST_LEAF_CONSTEXPR static int load( int err_id, F && f ) noexcept
+        BOOST_LEAF_CONSTEXPR static int load_( int err_id, F && f ) noexcept
         {
-            return accumulate_slot(err_id, std::forward<F>(f));
+            return accumulate_slot<false>(err_id, std::forward<F>(f));
         }
     };
 }
@@ -437,6 +595,7 @@ namespace leaf_detail
 ////////////////////////////////////////
 
 #if BOOST_LEAF_CFG_STD_SYSTEM_ERROR
+
 namespace leaf_detail
 {
     class leaf_category final: public std::error_category
@@ -444,7 +603,7 @@ namespace leaf_detail
         bool equivalent( int,  std::error_condition const & ) const noexcept final override { return false; }
         bool equivalent( std::error_code const &, int ) const noexcept final override { return false; }
         char const * name() const noexcept final override { return "LEAF error"; }
-        std::string message( int condition ) const final override { return name(); }
+        std::string message( int ) const final override { return name(); }
     public:
         ~leaf_category() noexcept final override { }
     };
@@ -471,7 +630,7 @@ namespace leaf_detail
             else
             {
                 err_id = new_id();
-                (void) load_slot(err_id, ec);
+                (void) load_slot<false>(err_id, ec);
                 return (err_id&~3)|1;
             }
         }
@@ -486,6 +645,7 @@ inline bool is_error_id( std::error_code const & ec ) noexcept
     BOOST_LEAF_ASSERT(!res || !ec.value() || ((ec.value()&3)==1));
     return res;
 }
+
 #endif
 
 ////////////////////////////////////////
@@ -538,12 +698,23 @@ public:
         return *this;
     }
 
+    template <class Item>
+    BOOST_LEAF_CONSTEXPR error_id load(Item && item) const noexcept
+    {
+        if (int err_id = value())
+        {
+            int const unused[] = { 42, leaf_detail::load_item<Item>::load_(err_id, std::forward<Item>(item)) };
+            (void)unused;
+        }
+        return *this;
+    }
+
     template <class... Item>
     BOOST_LEAF_CONSTEXPR error_id load( Item && ... item ) const noexcept
     {
         if( int err_id = value() )
         {
-            int const unused[ ] = { 42, leaf_detail::load_item<Item>::load(err_id, std::forward<Item>(item))... };
+            int const unused[ ] = { 42, leaf_detail::load_item<Item>::load_(err_id, std::forward<Item>(item))... };
             (void) unused;
         }
         return *this;
@@ -551,13 +722,8 @@ public:
 
     BOOST_LEAF_CONSTEXPR int value() const noexcept
     {
-        if( int v = value_ )
-        {
-            BOOST_LEAF_ASSERT((v&3)==1);
-            return (v&~3)|1;
-        }
-        else
-            return 0;
+        BOOST_LEAF_ASSERT(value_==0 || ((value_&3)==1));
+        return value_;
     }
 
     BOOST_LEAF_CONSTEXPR explicit operator bool() const noexcept
@@ -581,7 +747,7 @@ public:
     }
 
     template <class CharT, class Traits>
-    friend std::basic_ostream<CharT, Traits> & operator<<( std::basic_ostream<CharT, Traits> & os, error_id x )
+    friend std::ostream & operator<<( std::basic_ostream<CharT, Traits> & os, error_id x )
     {
         return os << x.value_;
     }
@@ -637,8 +803,15 @@ public:
     virtual void deactivate() noexcept = 0;
     virtual void propagate( error_id ) noexcept = 0;
     virtual bool is_active() const noexcept = 0;
-    inline virtual void print( std::ostream & ) const { };
+    virtual void print( std::ostream & ) const { };
     error_id captured_id_;
+
+    template <class CharT, class Traits>
+    friend std::ostream & operator<<( std::basic_ostream<CharT, Traits> & os, polymorphic_context const & ctx )
+    {
+        ctx.print(os);
+        return os;
+    }
 };
 
 #if BOOST_LEAF_CFG_CAPTURE
