@@ -15,17 +15,17 @@
 
 #include <boost/mp11.hpp>
 #include <boost/assert.hpp>
+#include <boost/assert/source_location.hpp>
 #include <boost/config.hpp>
-#include <boost/detail/workaround.hpp>
-#include <boost/cstdint.hpp>
+#include <boost/config/workaround.hpp>
 #include <cstddef>
 #include <type_traits>
 #include <exception>
-#include <initializer_list>
 #include <utility>
-#include <functional> // std::hash
-#include <cstdint>
+#include <typeindex> // std::hash
 #include <iosfwd>
+#include <cstdint>
+#include <cerrno>
 
 //
 
@@ -685,7 +685,20 @@ template<class T1, class... T> union variant_storage_impl<mp11::mp_true, T1, T..
 
     template<std::size_t I, class... A> BOOST_CXX14_CONSTEXPR void emplace_impl( mp11::mp_true, mp11::mp_size_t<I>, A&&... a )
     {
+#if defined(BOOST_GCC) && (__GNUC__ >= 7)
+# pragma GCC diagnostic push
+// False positive in at least GCC 7 and GCC 10 ASAN triggered by monostate (via result<void>)
+# pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
+#if __GNUC__ >= 12
+// False positive in at least GCC 12 and GCC 13 ASAN and -Og triggered by monostate (via result<void>)
+# pragma GCC diagnostic ignored "-Wuninitialized"
+#endif
+#endif
         *this = variant_storage_impl( mp11::mp_size_t<I>(), std::forward<A>(a)... );
+
+#if defined(BOOST_GCC) && (__GNUC__ >= 7)
+# pragma GCC diagnostic pop
+#endif
     }
 
     template<std::size_t I, class... A> BOOST_CXX14_CONSTEXPR void emplace( mp11::mp_size_t<I>, A&&... a )
@@ -908,6 +921,11 @@ template<class... T> struct variant_base_impl<true, true, T...>
 
         this->emplace_impl<J, U>( std::is_nothrow_constructible<U, A&&...>(), std::forward<A>(a)... );
     }
+
+    static constexpr bool uses_double_storage() noexcept
+    {
+        return false;
+    }
 };
 
 // trivially destructible, double buffered
@@ -966,6 +984,11 @@ template<class... T> struct variant_base_impl<true, false, T...>
 
         ix_ = J * 2 + i2;
     }
+
+    static constexpr bool uses_double_storage() noexcept
+    {
+        return true;
+    }
 };
 
 // not trivially destructible, single buffered
@@ -1056,6 +1079,11 @@ template<class... T> struct variant_base_impl<false, true, T...>
         st_.emplace( mp11::mp_size_t<J>(), std::move(tmp) );
         ix_ = J;
     }
+
+    static constexpr bool uses_double_storage() noexcept
+    {
+        return false;
+    }
 };
 
 // not trivially destructible, double buffered
@@ -1181,6 +1209,11 @@ template<class... T> struct variant_base_impl<false, false, T...>
 
         ix_ = J * 2 + i2;
     }
+
+    static constexpr bool uses_double_storage() noexcept
+    {
+        return true;
+    }
 };
 
 } // namespace detail
@@ -1691,6 +1724,8 @@ public:
 
     using variant_base::index;
 
+    using variant_base::uses_double_storage;
+
     // swap
 
 private:
@@ -2285,8 +2320,8 @@ namespace detail
 
 inline std::size_t hash_value_impl_( mp11::mp_true, std::size_t index, std::size_t value )
 {
-    boost::ulong_long_type hv = ( boost::ulong_long_type( 0xCBF29CE4 ) << 32 ) + 0x84222325;
-    boost::ulong_long_type const prime = ( boost::ulong_long_type( 0x00000100 ) << 32 ) + 0x000001B3;
+    unsigned long long hv = 0xCBF29CE484222325ull;
+    unsigned long long const prime = 0x100000001B3ull;
 
     hv ^= index;
     hv *= prime;
@@ -2392,6 +2427,109 @@ template<> struct hash< ::boost::variant2::monostate >
 
 } // namespace std
 
+// JSON support
+
+namespace boost
+{
+namespace json
+{
+
+class value;
+
+struct value_from_tag;
+
+template<class T>
+void value_from( T&& t, value& jv );
+
+template<class T>
+struct try_value_to_tag;
+
+template<class T1, class T2>
+struct result_for;
+
+template<class T>
+typename result_for<T, value>::type
+try_value_to( value const & jv );
+
+template<class T>
+typename result_for<T, value>::type
+result_from_errno( int e, boost::source_location const* loc ) noexcept;
+
+template<class T> struct is_null_like;
+
+template<> struct is_null_like<variant2::monostate>: std::true_type {};
+
+} // namespace json
+
+namespace variant2
+{
+
+namespace detail
+{
+
+struct tag_invoke_L1
+{
+    boost::json::value& v;
+
+#if defined(BOOST_MSVC) && BOOST_MSVC / 10 == 191
+    // msvc-14.1 with /permissive- needs this
+    explicit tag_invoke_L1( boost::json::value& v_ ): v( v_ ) {}
+#endif
+
+    template<class T> void operator()( T const& t ) const
+    {
+        boost::json::value_from( t, v );
+    }
+};
+
+} // namespace detail
+
+template<class... T>
+    void tag_invoke( boost::json::value_from_tag const&, boost::json::value& v, variant<T...> const & w )
+{
+    visit( detail::tag_invoke_L1{ v }, w );
+}
+
+namespace detail
+{
+
+template<class V> struct tag_invoke_L2
+{
+    boost::json::value const& v;
+    typename boost::json::result_for<V, boost::json::value>::type& r;
+
+    template<class I> void operator()( I /*i*/ ) const
+    {
+        if( !r )
+        {
+            using Ti = mp11::mp_at_c<V, I::value>;
+            auto r2 = boost::json::try_value_to<Ti>( v );
+
+            if( r2 )
+            {
+                r.emplace( in_place_index_t<I::value>{}, std::move( *r2 ) );
+            }
+        }
+    }
+};
+
+} // namespace detail
+
+template<class... T>
+    typename boost::json::result_for<variant<T...>, boost::json::value>::type
+    tag_invoke( boost::json::try_value_to_tag<variant<T...>> const&, boost::json::value const& v )
+{
+    static constexpr boost::source_location loc = BOOST_CURRENT_LOCATION;
+    auto r = boost::json::result_from_errno< variant<T...> >( EINVAL, &loc );
+
+    mp11::mp_for_each<mp11::mp_iota_c<sizeof...(T)>>( detail::tag_invoke_L2< variant<T...> >{ v, r } );
+
+    return r;
+}
+
+} // namespace variant2
+} // namespace boost
+
 #undef BOOST_VARIANT2_CX14_ASSERT
 
 #if defined(_MSC_VER) && _MSC_VER < 1910
